@@ -1,6 +1,5 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Big = imports.gi.Big;
 const Clutter = imports.gi.Clutter;
 const Gdk = imports.gi.Gdk;
 const Lang = imports.lang;
@@ -14,15 +13,7 @@ const St = imports.gi.St;
 const Main = imports.ui.main;
 const Tweener = imports.ui.tweener;
 
-const POPUP_ARROW_COLOR = new Clutter.Color();
-POPUP_ARROW_COLOR.from_pixel(0xffffffff);
-const POPUP_UNFOCUSED_ARROW_COLOR = new Clutter.Color();
-POPUP_UNFOCUSED_ARROW_COLOR.from_pixel(0x808080ff);
-const TRANSPARENT_COLOR = new Clutter.Color();
-TRANSPARENT_COLOR.from_pixel(0x00000000);
-
 const POPUP_APPICON_SIZE = 96;
-const POPUP_LIST_SPACING = 8;
 const POPUP_SCROLL_TIME = 0.10; // seconds
 
 const DISABLE_HOVER_TIMEOUT = 500; // milliseconds
@@ -43,7 +34,8 @@ function AltTabPopup() {
 
 AltTabPopup.prototype = {
     _init : function() {
-        this.actor = new Shell.GenericContainer({ reactive: true });
+        this.actor = new Shell.GenericContainer({ name: 'altTabPopup',
+                                                    reactive: true });
 
         this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
         this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
@@ -54,7 +46,7 @@ AltTabPopup.prototype = {
         this._haveModal = false;
 
         this._currentApp = 0;
-        this._currentWindow = 0;
+        this._currentWindow = -1;
         this._thumbnailTimeoutId = 0;
         this._motionTimeoutId = 0;
 
@@ -62,7 +54,7 @@ AltTabPopup.prototype = {
         // the switcher appears underneath the current pointer location
         this._disableHover();
 
-        global.stage.add_actor(this.actor);
+        Main.uiGroup.add_actor(this.actor);
     },
 
     _getPreferredWidth: function (actor, forHeight, alloc) {
@@ -79,12 +71,18 @@ AltTabPopup.prototype = {
         let childBox = new Clutter.ActorBox();
         let focus = global.get_focus_monitor();
 
+        let leftPadding = this.actor.get_theme_node().get_padding(St.Side.LEFT);
+        let rightPadding = this.actor.get_theme_node().get_padding(St.Side.RIGHT);
+        let bottomPadding = this.actor.get_theme_node().get_padding(St.Side.BOTTOM);
+        let vPadding = this.actor.get_theme_node().get_vertical_padding();
+        let hPadding = leftPadding + rightPadding;
+
         // Allocate the appSwitcher
         // We select a size based on an icon size that does not overflow the screen
-        let [childMinHeight, childNaturalHeight] = this._appSwitcher.actor.get_preferred_height(focus.width - POPUP_LIST_SPACING * 2);
+        let [childMinHeight, childNaturalHeight] = this._appSwitcher.actor.get_preferred_height(focus.width - hPadding);
         let [childMinWidth, childNaturalWidth] = this._appSwitcher.actor.get_preferred_width(childNaturalHeight);
-        childBox.x1 = Math.max(POPUP_LIST_SPACING, focus.x + Math.floor((focus.width - childNaturalWidth) / 2));
-        childBox.x2 = Math.min(childBox.x1 + focus.width - POPUP_LIST_SPACING * 2, childBox.x1 + childNaturalWidth);
+        childBox.x1 = Math.max(focus.x + leftPadding, focus.x + Math.floor((focus.width - childNaturalWidth) / 2));
+        childBox.x2 = Math.min(childBox.x1 + focus.width - hPadding, childBox.x1 + childNaturalWidth);
         childBox.y1 = focus.y + Math.floor((focus.height - childNaturalHeight) / 2);
         childBox.y2 = childBox.y1 + childNaturalHeight;
         this._appSwitcher.actor.allocate(childBox, flags);
@@ -94,20 +92,26 @@ AltTabPopup.prototype = {
         // those calculations
         if (this._thumbnails) {
             let icon = this._appIcons[this._currentApp].actor;
+            // Force a stage relayout to make sure we get the correct position
+            global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, 0, 0);
             let [posX, posY] = icon.get_transformed_position();
             let thumbnailCenter = posX + icon.width / 2;
             let [childMinWidth, childNaturalWidth] = this._thumbnails.actor.get_preferred_width(-1);
-            childBox.x1 = Math.max(POPUP_LIST_SPACING, Math.floor(thumbnailCenter - childNaturalWidth / 2));
-            if (childBox.x1 + childNaturalWidth > focus.width - POPUP_LIST_SPACING * 2) {
-                let offset = childBox.x1 + childNaturalWidth - focus.width + POPUP_LIST_SPACING * 2;
-                childBox.x1 = Math.max(POPUP_LIST_SPACING, childBox.x1 - offset - POPUP_LIST_SPACING * 2);
+            childBox.x1 = Math.max(focus.x + leftPadding, Math.floor(thumbnailCenter - childNaturalWidth / 2));
+            if (childBox.x1 + childNaturalWidth > focus.x + focus.width - hPadding) {
+                let offset = childBox.x1 + childNaturalWidth - focus.width + hPadding;
+                childBox.x1 = Math.max(focus.x + leftPadding, childBox.x1 - offset - hPadding);
             }
 
+            let [found, spacing] = this.actor.get_theme_node().get_length('spacing', false);
+            if (!found)
+                spacing = 0;
+
             childBox.x2 = childBox.x1 +  childNaturalWidth;
-            if (childBox.x2 > focus.width - POPUP_LIST_SPACING)
-                childBox.x2 = focus.width - POPUP_LIST_SPACING;
-            childBox.y1 = this._appSwitcher.actor.allocation.y2 + POPUP_LIST_SPACING * 2;
-            this._thumbnails.addClones(focus.height - POPUP_LIST_SPACING - childBox.y1);
+            if (childBox.x2 > focus.x + focus.width - rightPadding)
+                childBox.x2 = focus.x + focus.width - rightPadding;
+            childBox.y1 = this._appSwitcher.actor.allocation.y2 + spacing;
+            this._thumbnails.addClones(focus.height - bottomPadding - childBox.y1);
             let [childMinHeight, childNaturalHeight] = this._thumbnails.actor.get_preferred_height(-1);
             childBox.y2 = childBox.y1 + childNaturalHeight;
             this._thumbnails.actor.allocate(childBox, flags);
@@ -116,7 +120,7 @@ AltTabPopup.prototype = {
 
     show : function(backward) {
         let tracker = Shell.WindowTracker.get_default();
-        let apps = tracker.get_running_apps ("");
+        let apps = tracker.get_running_apps ('');
 
         if (!apps.length)
             return false;
@@ -160,8 +164,9 @@ AltTabPopup.prototype = {
                     this._select(0, 1, true);
                 else
                     this._select(1);
-            } else
+            } else {
                 this._select(1);
+            }
         }
 
         // There's a race condition; if the user released Alt before
@@ -169,7 +174,7 @@ AltTabPopup.prototype = {
         // https://bugzilla.gnome.org/show_bug.cgi?id=596695 for
         // details.) So we check now. (Have to do this after updating
         // selection.)
-        let mods = global.get_modifier_keys();
+        let [x, y, mods] = global.get_pointer();
         if (!(mods & Gdk.ModifierType.MOD1_MASK)) {
             this._finish();
             return false;
@@ -186,10 +191,16 @@ AltTabPopup.prototype = {
     },
 
     _nextWindow : function() {
+        // We actually want the second window if we're in the unset state
+        if (this._currentWindow == -1)
+            this._currentWindow = 0;
         return mod(this._currentWindow + 1,
                    this._appIcons[this._currentApp].cachedWindows.length);
     },
     _previousWindow : function() {
+        // Also assume second window here
+        if (this._currentWindow == -1)
+            this._currentWindow = 1;
         return mod(this._currentWindow - 1,
                    this._appIcons[this._currentApp].cachedWindows.length);
     },
@@ -209,7 +220,7 @@ AltTabPopup.prototype = {
             this.destroy();
         else if (this._thumbnailsFocused) {
             if (keysym == Clutter.Tab) {
-                if (shift && this._currentWindow == 0)
+                if (shift && (this._currentWindow == 0 || this._currentWindow == -1))
                     this._select(this._previousApp());
                 else if (!shift && this._currentWindow == this._appIcons[this._currentApp].cachedWindows.length - 1)
                     this._select(this._nextApp());
@@ -229,7 +240,7 @@ AltTabPopup.prototype = {
             else if (keysym == Clutter.Right || keysym == Clutter.d)
                 this._select(this._nextApp());
             else if (keysym == Clutter.Down || keysym == Clutter.s)
-                this._select(this._currentApp, this._currentWindow);
+                this._select(this._currentApp, 0);
         }
 
         return true;
@@ -248,7 +259,7 @@ AltTabPopup.prototype = {
         let direction = event.get_scroll_direction();
         if (direction == Clutter.ScrollDirection.UP) {
             if (this._thumbnailsFocused) {
-                if (this._currentWindow == 0)
+                if (this._currentWindow == 0 || this._currentWindow == -1)
                     this._select(this._previousApp());
                 else
                     this._select(this._currentApp, this._previousWindow());
@@ -282,10 +293,17 @@ AltTabPopup.prototype = {
     _appActivated : function(appSwitcher, n) {
         // If the user clicks on the selected app, activate the
         // selected window; otherwise (eg, they click on an app while
-        // !mouseActive) activate the first window of the clicked-on
-        // app.
-        let window = (n == this._currentApp) ? this._currentWindow : 0;
-        Main.activateWindow(this._appIcons[n].cachedWindows[window]);
+        // !mouseActive) activate the the clicked-on app.
+        if (n == this._currentApp) {
+            let window;
+            if (this._currentWindow >= 0)
+                window = this._appIcons[this._currentApp].cachedWindows[this._currentWindow];
+            else
+                window = null;
+            this._appIcons[this._currentApp].app.activate_window(window, global.get_current_time());
+        } else {
+            this._appIcons[n].app.activate_window(null, global.get_current_time());
+        }
         this.destroy();
     },
 
@@ -297,7 +315,8 @@ AltTabPopup.prototype = {
     },
 
     _windowActivated : function(thumbnailList, n) {
-        Main.activateWindow(this._appIcons[this._currentApp].cachedWindows[n]);
+        let appIcon = this._appIcons[this._currentApp];
+        Main.activateWindow(appIcon.cachedWindows[n]);
         this.destroy();
     },
 
@@ -324,8 +343,11 @@ AltTabPopup.prototype = {
 
     _finish : function() {
         let app = this._appIcons[this._currentApp];
-        let window = app.cachedWindows[this._currentWindow];
-        Main.activateWindow(window);
+        if (this._currentWindow >= 0) {
+            Main.activateWindow(app.cachedWindows[this._currentWindow]);
+        } else {
+            app.app.activate_window(null, global.get_current_time());
+        }
         this.destroy();
     },
 
@@ -390,7 +412,7 @@ AltTabPopup.prototype = {
         this._thumbnailsFocused = (window != null) && !forceAppFocus;
 
         this._currentApp = app;
-        this._currentWindow = window ? window : 0;
+        this._currentWindow = window ? window : -1;
         this._appSwitcher.highlight(app, this._thumbnailsFocused);
 
         if (window != null) {
@@ -402,18 +424,23 @@ AltTabPopup.prototype = {
                    !forceAppFocus) {
             this._thumbnailTimeoutId = Mainloop.timeout_add (
                 THUMBNAIL_POPUP_TIME,
-                Lang.bind(this, function () {
-                              this._select(this._currentApp, 0, true);
-                              return false;
-                          }));
+                Lang.bind(this, this._timeoutPopupThumbnails));
         }
     },
 
+    _timeoutPopupThumbnails: function() {
+        if (!this._thumbnails)
+            this._createThumbnails();
+        this._thumbnailTimeoutId = 0;
+        this._thumbnailsFocused = false;
+        return false;
+    },
+
     _destroyThumbnails : function() {
         Tweener.addTween(this._thumbnails.actor,
                          { opacity: 0,
                            time: THUMBNAIL_FADE_TIME,
-                           transition: "easeOutQuad",
+                           transition: 'easeOutQuad',
                            onComplete: function() { this.destroy(); }
                          });
         this._thumbnails = null;
@@ -430,7 +457,7 @@ AltTabPopup.prototype = {
         Tweener.addTween(this._thumbnails.actor,
                          { opacity: 255,
                            time: THUMBNAIL_FADE_TIME,
-                           transition: "easeOutQuad"
+                           transition: 'easeOutQuad'
                          });
     }
 };
@@ -441,12 +468,19 @@ function SwitcherList(squareItems) {
 
 SwitcherList.prototype = {
     _init : function(squareItems) {
-        this.actor = new St.BoxLayout({ style_class: 'switcher-list' });
+        this.actor = new Shell.GenericContainer({ style_class: 'switcher-list' });
+        this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
+        this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
+        this.actor.connect('allocate', Lang.bind(this, this._allocateTop));
 
         // Here we use a GenericContainer so that we can force all the
         // children except the separator to have the same width.
-        this._list = new Shell.GenericContainer();
-        this._list.spacing = POPUP_LIST_SPACING;
+        this._list = new Shell.GenericContainer({ style_class: 'switcher-list-item-container' });
+        this._list.spacing = 0;
+        this._list.connect('style-changed', Lang.bind(this, function() {
+                                                        let [found, spacing] = this._list.get_theme_node().get_length('spacing', false);
+                                                        this._list.spacing = (found) ? spacing : 0;
+                                                     }));
 
         this._list.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
         this._list.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
@@ -462,26 +496,72 @@ SwitcherList.prototype = {
         this.actor.add_actor(this._rightGradient);
 
         // Those arrows indicate whether scrolling in one direction is possible
-        this._leftArrow = new St.DrawingArea();
+        this._leftArrow = new St.DrawingArea({ style_class: 'switcher-arrow',
+                                               pseudo_class: 'highlighted' });
         this._leftArrow.connect('repaint', Lang.bind(this,
                                             function (area) {
-                                                Shell.draw_box_pointer(area, Shell.PointerDirection.LEFT, TRANSPARENT_COLOR, POPUP_ARROW_COLOR);
+                                                Shell.draw_box_pointer(area, Shell.PointerDirection.LEFT);
                                             }));
 
-        this._rightArrow = new St.DrawingArea();
+        this._rightArrow = new St.DrawingArea({ style_class: 'switcher-arrow',
+                                                pseudo_class: 'highlighted' });
         this._rightArrow.connect('repaint', Lang.bind(this,
                                             function (area) {
-                                                Shell.draw_box_pointer(area, Shell.PointerDirection.RIGHT, TRANSPARENT_COLOR, POPUP_ARROW_COLOR);
+                                                Shell.draw_box_pointer(area, Shell.PointerDirection.RIGHT);
                                             }));
 
-        this._leftGradient.add_actor(this._leftArrow);
-        this._rightGradient.add_actor(this._rightArrow);
+        this.actor.add_actor(this._leftArrow);
+        this.actor.add_actor(this._rightArrow);
 
         this._items = [];
         this._highlighted = -1;
         this._separator = null;
         this._squareItems = squareItems;
-        this._scrollable = false;
+        this._minSize = 0;
+        this._scrollableRight = true;
+        this._scrollableLeft = false;
+    },
+
+    _allocateTop: function(actor, box, flags) {
+        let leftPadding = this.actor.get_theme_node().get_padding(St.Side.LEFT);
+        let rightPadding = this.actor.get_theme_node().get_padding(St.Side.RIGHT);
+
+        let childBox = new Clutter.ActorBox();
+        let scrollable = this._minSize > box.x2 - box.x1;
+
+        this._clipBin.allocate(box, flags);
+
+        childBox.x1 = 0;
+        childBox.y1 = 0;
+        childBox.x2 = this._leftGradient.width;
+        childBox.y2 = this.actor.height;
+        this._leftGradient.allocate(childBox, flags);
+        this._leftGradient.opacity = (this._scrollableLeft && scrollable) ? 255 : 0;
+
+        childBox.x1 = (this.actor.allocation.x2 - this.actor.allocation.x1) - this._rightGradient.width;
+        childBox.y1 = 0;
+        childBox.x2 = childBox.x1 + this._rightGradient.width;
+        childBox.y2 = this.actor.height;
+        this._rightGradient.allocate(childBox, flags);
+        this._rightGradient.opacity = (this._scrollableRight && scrollable) ? 255 : 0;
+
+        let arrowWidth = Math.floor(leftPadding / 3);
+        let arrowHeight = arrowWidth * 2;
+        childBox.x1 = leftPadding / 2;
+        childBox.y1 = this.actor.height / 2 - arrowWidth;
+        childBox.x2 = childBox.x1 + arrowWidth;
+        childBox.y2 = childBox.y1 + arrowHeight;
+        this._leftArrow.allocate(childBox, flags);
+        this._leftArrow.opacity = this._leftGradient.opacity;
+
+        arrowWidth = Math.floor(rightPadding / 3);
+        arrowHeight = arrowWidth * 2;
+        childBox.x1 = this.actor.width - arrowWidth - rightPadding / 2;
+        childBox.y1 = this.actor.height / 2 - arrowWidth;
+        childBox.x2 = childBox.x1 + arrowWidth;
+        childBox.y2 = childBox.y1 + arrowHeight;
+        this._rightArrow.allocate(childBox, flags);
+        this._rightArrow.opacity = this._rightGradient.opacity;
     },
 
     addItem : function(item) {
@@ -525,8 +605,7 @@ SwitcherList.prototype = {
         let itemSize = this._items[index].allocation.x2 - this._items[index].allocation.x1;
         let [posX, posY] = this._items[index].get_transformed_position();
         posX += this.actor.x;
-
-        if (posX + itemSize > monitor.width)
+        if (posX + itemSize > monitor.width + monitor.x)
             this._scrollToRight();
         else if (posX < 0)
             this._scrollToLeft();
@@ -535,28 +614,33 @@ SwitcherList.prototype = {
 
     _scrollToLeft : function() {
         let x = this._items[this._highlighted].allocation.x1;
-        this._rightGradient.show();
+        this._scrollableRight = true;
         Tweener.addTween(this._list, { anchor_x: x,
                                         time: POPUP_SCROLL_TIME,
                                         transition: 'easeOutQuad',
                                         onComplete: Lang.bind(this, function () {
-                                                                        if (this._highlighted == 0)
-                                                                            this._leftGradient.hide();
+                                                                        if (this._highlighted == 0) {
+                                                                            this._scrollableLeft = false;
+                                                                            this.actor.queue_relayout();
+                                                                        }
                                                              })
                         });
     },
 
     _scrollToRight : function() {
+        this._scrollableLeft = true;
         let monitor = global.get_focus_monitor();
         let padding = this.actor.get_theme_node().get_horizontal_padding();
-        let x = this._items[this._highlighted].allocation.x2 - monitor.width + padding + POPUP_LIST_SPACING * 2;
-        this._leftGradient.show();
+        let parentPadding = this.actor.get_parent().get_theme_node().get_horizontal_padding();
+        let x = this._items[this._highlighted].allocation.x2 - monitor.width + padding + parentPadding;
         Tweener.addTween(this._list, { anchor_x: x,
                                         time: POPUP_SCROLL_TIME,
                                         transition: 'easeOutQuad',
                                         onComplete: Lang.bind(this, function () {
-                                                                        if (this._highlighted == this._items.length - 1)
-                                                                            this._rightGradient.hide();
+                                                                        if (this._highlighted == this._items.length - 1) {
+                                                                            this._scrollableRight = false;
+                                                                            this.actor.queue_relayout();
+                                                                        }
                                                              })
                         });
     },
@@ -600,6 +684,7 @@ SwitcherList.prototype = {
         let totalSpacing = this._list.spacing * (this._items.length - 1);
         alloc.min_size = this._items.length * maxChildMin + separatorWidth + totalSpacing;
         alloc.natural_size = alloc.min_size;
+        this._minSize = alloc.min_size;
     },
 
     _getPreferredHeight: function (actor, forWidth, alloc) {
@@ -642,11 +727,14 @@ SwitcherList.prototype = {
         let childBox = new Clutter.ActorBox();
 
         let focus = global.get_focus_monitor();
-        if (this.actor.allocation.x2 == focus.width - POPUP_LIST_SPACING) {
+        let parentRightPadding = this.actor.get_parent().get_theme_node().get_padding(St.Side.RIGHT);
+        if (this.actor.allocation.x2 == focus.x + focus.width - parentRightPadding) {
             if (this._squareItems)
                 childWidth = childHeight;
-            else
-                childWidth = children[0].get_preferred_width(childHeight)[0];
+            else {
+                let [childMin, childNat] = children[0].get_preferred_width(childHeight);
+                childWidth = childMin;
+            }
         }
 
         for (let i = 0; i < children.length; i++) {
@@ -679,36 +767,6 @@ SwitcherList.prototype = {
         let topPadding = this.actor.get_theme_node().get_padding(St.Side.TOP);
         let bottomPadding = this.actor.get_theme_node().get_padding(St.Side.BOTTOM);
 
-        // Show the arrows and gradients when scrolling is needed
-        if (children[children.length - 1].allocation.x2 > this.actor.width - leftPadding - rightPadding && !this._scrollable) {
-            this._leftGradient.set_height(this.actor.height);
-            this._leftGradient.x = this.actor.x;
-            this._leftGradient.y = this.actor.y;
-
-            this._rightGradient.set_height(this.actor.height);
-            this._rightGradient.x = this.actor.x + (this.actor.allocation.x2 - this.actor.allocation.x1) - this._rightGradient.width;
-            this._rightGradient.y = this.actor.y;
-
-            let arrowWidth = Math.floor(leftPadding / 3);
-            let arrowHeight = arrowWidth * 2;
-            this._leftArrow.set_size(arrowWidth, arrowHeight);
-            this._leftArrow.set_position(POPUP_LIST_SPACING, this.actor.height / 2 - arrowWidth);
-
-            arrowWidth = Math.floor(rightPadding / 3);
-            arrowHeight = arrowWidth * 2;
-            this._rightArrow.set_size(arrowWidth, arrowHeight);
-            this._rightArrow.set_position(this._rightGradient.width - arrowHeight, this.actor.height / 2 - arrowWidth);
-
-            this._scrollable = true;
-
-            this._leftGradient.hide();
-            this._rightGradient.show();
-        }
-        else if (!this._scrollable){
-            this._leftGradient.hide();
-            this._rightGradient.hide();
-        }
-
         // Clip the area for scrolling
         this._clipBin.set_clip(0, -topPadding, (this.actor.allocation.x2 - this.actor.allocation.x1) - leftPadding - rightPadding, this.actor.height + bottomPadding);
     }
@@ -723,7 +781,7 @@ function AppIcon(app) {
 AppIcon.prototype = {
     _init: function(app) {
         this.app = app;
-        this.actor = new St.BoxLayout({ style_class: "alt-tab-app",
+        this.actor = new St.BoxLayout({ style_class: 'alt-tab-app',
                                          vertical: true });
         this.icon = null;
         this._iconBin = new St.Bin();
@@ -783,6 +841,8 @@ AppSwitcher.prototype = {
 
     _getPreferredHeight: function (actor, forWidth, alloc) {
         let j = 0;
+        let height = 0;
+
         while(this._items.length > 1 && this._items[j].style_class != 'item-box') {
                 j++;
         }
@@ -793,16 +853,11 @@ AppSwitcher.prototype = {
         if (this._separator)
            totalSpacing += this._separator.width + this._list.spacing;
 
-        // We just assume the whole screen here due to weirdness happing with the passed width
-        let focus = global.get_focus_monitor();
-        let availWidth = focus.width - POPUP_LIST_SPACING * 2 - this.actor.get_theme_node().get_horizontal_padding();
-        let height = 0;
-
         for(let i =  0; i < iconSizes.length; i++) {
                 this._iconSize = iconSizes[i];
                 height = iconSizes[i] + iconSpacing;
                 let w = height * this._items.length + totalSpacing;
-                if (w <= availWidth)
+                if (w <= forWidth)
                         break;
         }
 
@@ -811,17 +866,17 @@ AppSwitcher.prototype = {
             height = iconSizes[0] + iconSpacing;
         }
 
-        alloc.min_size = height;
-        alloc.natural_size = height;
-    },
-
-    _allocate: function (actor, box, flags) {
         for(let i = 0; i < this.icons.length; i++) {
             if (this.icons[i].icon != null)
                 break;
             this.icons[i].set_size(this._iconSize);
         }
 
+        alloc.min_size = height;
+        alloc.natural_size = height;
+    },
+
+    _allocate: function (actor, box, flags) {
         // Allocate the main list items
         SwitcherList.prototype._allocate.call(this, actor, box, flags);
 
@@ -846,16 +901,13 @@ AppSwitcher.prototype = {
     // thumbnails are visible (ie, when the app icon is supposed to be
     // in justOutline mode). Apps with multiple windows will normally
     // show a dim arrow, but show a bright arrow when they are
-    // highlighted; their redraw handler will use the right color
-    // based on this._curApp; we just need to do a queue_relayout() to
-    // force it to redraw. (queue_redraw() doesn't work because
-    // ShellDrawingArea only redraws on allocate.)
+    // highlighted.
     highlight : function(n, justOutline) {
         if (this._curApp != -1) {
             if (this.icons[this._curApp].cachedWindows.length == 1)
                 this._arrows[this._curApp].hide();
             else
-                this._arrows[this._curApp].queue_relayout();
+                this._arrows[this._curApp].remove_style_pseudo_class('highlighted');
         }
 
         SwitcherList.prototype.highlight.call(this, n, justOutline);
@@ -865,7 +917,7 @@ AppSwitcher.prototype = {
             if (justOutline && this.icons[this._curApp].cachedWindows.length == 1)
                 this._arrows[this._curApp].show();
             else
-                this._arrows[this._curApp].queue_relayout();
+                this._arrows[this._curApp].add_style_pseudo_class('highlighted');
         }
     },
 
@@ -874,12 +926,10 @@ AppSwitcher.prototype = {
         this.addItem(appIcon.actor);
 
         let n = this._arrows.length;
-        let arrow = new St.DrawingArea();
+        let arrow = new St.DrawingArea({ style_class: 'switcher-arrow' });
         arrow.connect('repaint', Lang.bind(this,
             function (area) {
-                Shell.draw_box_pointer(area, Shell.PointerDirection.DOWN,
-                                       TRANSPARENT_COLOR,
-                                       this._curApp == n ? POPUP_ARROW_COLOR : POPUP_UNFOCUSED_ARROW_COLOR);
+                Shell.draw_box_pointer(area, Shell.PointerDirection.DOWN);
             }));
         this._list.add_actor(arrow);
         this._arrows.push(arrow);
@@ -914,7 +964,7 @@ ThumbnailList.prototype = {
 
         let activeWorkspace = global.screen.get_active_workspace();
 
-        // We fake the value of "separatorAdded" when the app has no window
+        // We fake the value of 'separatorAdded' when the app has no window
         // on the current workspace, to avoid displaying a useless separator in
         // that case.
         let separatorAdded = windows.length == 0 || windows[0].get_workspace() != activeWorkspace;
@@ -930,10 +980,10 @@ ThumbnailList.prototype = {
               separatorAdded = true;
             }
 
-            let box = new St.BoxLayout({ style_class: "thumbnail-box",
+            let box = new St.BoxLayout({ style_class: 'thumbnail-box',
                                          vertical: true });
 
-            let bin = new St.Bin({ style_class: "thumbnail" });
+            let bin = new St.Bin({ style_class: 'thumbnail' });
 
             box.add_actor(bin);
             this._thumbnailBins.push(bin);
