@@ -5,6 +5,7 @@ const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
 const Shell = imports.gi.Shell;
+const Signals = imports.signals;
 
 const Main = imports.ui.main;
 const Params = imports.misc.params;
@@ -13,6 +14,19 @@ const Params = imports.misc.params;
 // normal mode (ie, outside the Overview), that surrounds the main
 // workspace content.
 
+const Visibility = {
+    FULL:       1,
+    FULLSCREEN: 2,
+    OVERVIEW:   3
+};
+
+const defaultParams = {
+    visibleInOverview: false,
+    visibleInFullscreen: false,
+    affectsStruts: true,
+    affectsInputRegion: true
+};
+
 function Chrome() {
     this._init();
 }
@@ -20,12 +34,13 @@ function Chrome() {
 Chrome.prototype = {
     _init: function() {
         // The group itself has zero size so it doesn't interfere with DND
-        this.actor = new Clutter.Group({ width: 0, height: 0 });
-        global.stage.add_actor(this.actor);
-        this.nonOverviewActor = new Clutter.Group();
-        this.actor.add_actor(this.nonOverviewActor);
+        this.actor = new Shell.GenericContainer({ width: 0, height: 0 });
+        Main.uiGroup.add_actor(this.actor);
+        this.actor.connect('allocate', Lang.bind(this, this._allocated));
 
-        this._obscuredByFullscreen = false;
+        this._inFullscreen = false;
+        this._inOverview = false;
+        this.visibility = Visibility.FULL;
 
         this._trackedActors = [];
 
@@ -44,6 +59,12 @@ Chrome.prototype = {
         this._queueUpdateRegions();
     },
 
+    _allocated: function(actor, box, flags) {
+        let children = this.actor.get_children();
+        for (let i = 0; i < children.length; i++)
+            children[i].allocate_preferred_size(flags);
+    },
+
     _verifyAncestry: function(actor, ancestor) {
         while (actor) {
             if (actor == ancestor)
@@ -67,19 +88,15 @@ Chrome.prototype = {
     //
     // If %visibleInOverview is %true in @params, @actor will remain
     // visible when the overview is brought up. Otherwise it will
-    // automatically be hidden. If %affectsStruts or %affectsInputRegion
-    // is %false, the actor will not have the indicated effect.
+    // automatically be hidden. Likewise, if %visibleInFullscreen is
+    // %true, the actor will be visible even when a fullscreen window
+    // should be covering it.
+    //
+    // If %affectsStruts or %affectsInputRegion is %false, the actor
+    // will not have the indicated effect.
     addActor: function(actor, params) {
-        params = Params.parse(params, { visibleInOverview: false,
-                                        affectsStruts: true,
-                                        affectsInputRegion: true });
-
-        if (params.visibleInOverview)
-            this.actor.add_actor(actor);
-        else
-            this.nonOverviewActor.add_actor(actor);
-
-        this._trackActor(actor, params.affectsInputRegion, params.affectsStruts);
+        this.actor.add_actor(actor);
+        this._trackActor(actor, params);
     },
 
     // trackActor:
@@ -89,13 +106,32 @@ Chrome.prototype = {
     // Tells the chrome to track @actor, which must be a descendant
     // of an actor added via addActor(). This can be used to extend the
     // struts or input region to cover specific children.
+    //
+    // @params can have any of the same values as in addActor(), though
+    // some possibilities don't make sense (eg, trying to have a
+    // %visibleInOverview child of a non-%visibleInOverview parent).
+    // By default, @actor has the same params as its chrome ancestor.
     trackActor: function(actor, params) {
-        if (!this._verifyAncestry(actor, this.actor))
+        let ancestor = actor.get_parent();
+        let index = this._findActor(ancestor);
+        while (ancestor && index == -1) {
+            ancestor = ancestor.get_parent();
+            index = this._findActor(ancestor);
+        }
+        if (!ancestor)
             throw new Error('actor is not a descendent of the chrome layer');
 
-        params = Params.parse(params, { affectsStruts: true,
-                                        affectsInputRegion: true });
-        this._trackActor(actor, params.affectsInputRegion, params.affectsStruts);
+        let ancestorData = this._trackedActors[index];
+        if (!params)
+            params = {};
+        // We can't use Params.parse here because we want to drop
+        // the extra values like ancestorData.actor
+        for (let prop in defaultParams) {
+            if (!params[prop])
+                params[prop] = ancestorData[prop];
+        }
+
+        this._trackActor(actor, params);
     },
 
     // untrackActor:
@@ -111,10 +147,7 @@ Chrome.prototype = {
     //
     // Removes @actor from the chrome layer
     removeActor: function(actor) {
-        if (actor.get_parent() == this.nonOverviewActor)
-            this.nonOverviewActor.remove_actor(actor);
-        else
-            this.actor.remove_actor(actor);
+        this.actor.remove_actor(actor);
         this._untrackActor(actor);
     },
 
@@ -127,16 +160,12 @@ Chrome.prototype = {
         return -1;
     },
 
-    _trackActor: function(actor, inputRegion, strut) {
-        let actorData;
-
+    _trackActor: function(actor, params) {
         if (this._findActor(actor) != -1)
             throw new Error('trying to re-track existing chrome actor');
 
-        actorData = { actor: actor,
-                      inputRegion: inputRegion,
-                      strut: strut };
-
+        let actorData = Params.parse(params, defaultParams);
+        actorData.actor = actor;
         actorData.visibleId = actor.connect('notify::visible',
                                             Lang.bind(this, this._queueUpdateRegions));
         actorData.allocationId = actor.connect('notify::allocation',
@@ -170,16 +199,40 @@ Chrome.prototype = {
             this._untrackActor(actor);
     },
 
+    _updateVisibility: function() {
+        for (let i = 0; i < this._trackedActors.length; i++) {
+            let actorData = this._trackedActors[i];
+            if (this._inOverview && !actorData.visibleInOverview)
+                this.actor.set_skip_paint(actorData.actor, true);
+            else if (this._inFullscreen && !actorData.visibleInFullscreen)
+                this.actor.set_skip_paint(actorData.actor, true);
+            else
+                this.actor.set_skip_paint(actorData.actor, false);
+        }
+
+        let newVisibility;
+        if (this._inOverview)
+            newVisibility = Visibility.OVERVIEW;
+        else if (this._inFullscreen)
+            newVisibility = Visibility.FULLSCREEN;
+        else
+            newVisibility = Visibility.FULL;
+
+        if (newVisibility != this.visibility) {
+            this.visibility = newVisibility;
+            this.emit('visibility-changed', this.visibility);
+        }
+    },
+
     _overviewShowing: function() {
-        this.actor.show();
-        this.nonOverviewActor.hide();
+        this._inOverview = true;
+        this._updateVisibility();
         this._queueUpdateRegions();
     },
 
     _overviewHidden: function() {
-        if (this._obscuredByFullscreen)
-            this.actor.hide();
-        this.nonOverviewActor.show();
+        this._inOverview = false;
+        this._updateVisibility();
         this._queueUpdateRegions();
     },
 
@@ -196,7 +249,7 @@ Chrome.prototype = {
         // The chrome layer should be visible unless there is a window
         // with layer FULLSCREEN, or a window with layer
         // OVERRIDE_REDIRECT that covers the whole screen.
-        // ("override_redirect" is not actually a layer above all
+        // ('override_redirect' is not actually a layer above all
         // other windows, but this seems to be how mutter treats it
         // currently...) If we wanted to be extra clever, we could
         // figure out when an OVERRIDE_REDIRECT window was trying to
@@ -205,13 +258,23 @@ Chrome.prototype = {
 
         // @windows is sorted bottom to top.
 
-        this._obscuredByFullscreen = false;
+        let wasInFullscreen = this._inFullscreen;
+        this._inFullscreen = false;
         for (let i = windows.length - 1; i > -1; i--) {
             let layer = windows[i].get_meta_window().get_layer();
+
+            // There are 3 cases we check here for:
+            // 1.) Monitor sized window
+            // 2.) Window with a position somewhere on the primary screen having the _NET_WM_FULLSCREEN flag set
+            // 3.) Window that is partly off screen (tries to hide its decorations) which might have negative coords
+            // We check for 1.) and 2.) by checking if the upper right corner is on the primary monitor, but avoid the case
+            // where it overlaps with the secondary screen (like window.x + window.width == primary.x + primary.width)
+            // For 3.) we just ignore negative values as they don't really make sense
+
             if (layer == Meta.StackLayer.FULLSCREEN) {
-                if (windows[i].x >= primary.x && windows[i].x <= primary.x + primary.width &&
-                    windows[i].y >= primary.y && windows[i].y <= primary.y + primary.height) {
-                        this._obscuredByFullscreen = true;
+                if (Math.max(windows[i].x, 0) >= primary.x && Math.max(windows[i].x, 0) < primary.x + primary.width &&
+                    Math.max(windows[i].y, 0) >= primary.y && Math.max(windows[i].y, 0) < primary.y + primary.height) {
+                        this._inFullscreen = true;
                         break;
                 }
             }
@@ -220,16 +283,15 @@ Chrome.prototype = {
                     windows[i].x + windows[i].width >= primary.x + primary.width &&
                     windows[i].y <= primary.y &&
                     windows[i].y + windows[i].height >= primary.y + primary.height) {
-                    this._obscuredByFullscreen = true;
+                    this._inFullscreen = true;
                     break;
                 }
             } else
                 break;
         }
 
-        let shouldBeVisible = !this._obscuredByFullscreen || Main.overview.visible;
-        if (this.actor.visible != shouldBeVisible) {
-            this.actor.visible = shouldBeVisible;
+        if (this._inFullscreen != wasInFullscreen) {
+            this._updateVisibility();
             this._queueUpdateRegions();
         }
     },
@@ -241,7 +303,7 @@ Chrome.prototype = {
 
         for (i = 0; i < this._trackedActors.length; i++) {
             let actorData = this._trackedActors[i];
-            if (!actorData.inputRegion && !actorData.strut)
+            if (!actorData.affectsInputRegion && !actorData.affectsStruts)
                 continue;
 
             let [x, y] = actorData.actor.get_transformed_position();
@@ -252,10 +314,12 @@ Chrome.prototype = {
             h = Math.round(h);
             let rect = new Meta.Rectangle({ x: x, y: y, width: w, height: h});
 
-            if (actorData.inputRegion && actorData.actor.get_paint_visibility())
+            if (actorData.affectsInputRegion &&
+                actorData.actor.get_paint_visibility() &&
+                !this.actor.get_skip_paint(actorData.actor))
                 rects.push(rect);
 
-            if (!actorData.strut)
+            if (!actorData.affectsStruts)
                 continue;
 
             // Metacity wants to know what side of the screen the
@@ -307,3 +371,4 @@ Chrome.prototype = {
         return false;
     }
 };
+Signals.addSignalMethods(Chrome.prototype);
