@@ -1,10 +1,12 @@
-/* -*- mode: js2; js2-basic-offset: 4; tab-width: 4; indent-tabs-mode: nil -*- */
+/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
 const Gtk = imports.gi.Gtk;
+const St = imports.gi.St;
 const Lang = imports.lang;
 const Signals = imports.signals;
 const Tweener = imports.ui.tweener;
+const Main = imports.ui.main;
 
 const Params = imports.misc.params;
 
@@ -21,7 +23,7 @@ function _getEventHandlerActor() {
         eventHandlerActor = new Clutter.Rectangle();
         eventHandlerActor.width = 0;
         eventHandlerActor.height = 0;
-        global.stage.add_actor(eventHandlerActor);
+        Main.uiGroup.add_actor(eventHandlerActor);
         // We connect to 'event' rather than 'captured-event' because the capturing phase doesn't happen
         // when you've grabbed the pointer.
         eventHandlerActor.connect('event',
@@ -61,13 +63,20 @@ _Draggable.prototype = {
     },
 
     _onButtonPress : function (actor, event) {
-        // FIXME: we should make sure it's button 1, but we can't currently
-        // check that from JavaScript
+        if (event.get_button() != 1)
+            return false;
+
         if (Tweener.getTweenCount(actor))
             return false;
 
         this._buttonDown = true;
-        this._grabActor();
+        // special case St.Clickable: grabbing the pointer would mess up the
+        // internal state, so we start the drag manually on hover change
+        if (this.actor instanceof St.Clickable)
+            this.actor.connect('notify::hover',
+                               Lang.bind(this, this._onClickableHoverChanged));
+        else
+            this._grabActor();
 
         let [stageX, stageY] = event.get_coords();
         this._dragStartX = stageX;
@@ -76,6 +85,15 @@ _Draggable.prototype = {
         return false;
     },
 
+    _onClickableHoverChanged: function(button) {
+        if (button.hover || !button.held)
+            return;
+
+        button.fake_release();
+        this.startDrag(this._dragStartX, this._dragStartY,
+                       global.get_current_time());
+    },
+
     _grabActor: function() {
         Clutter.grab_pointer(this.actor);
         this._onEventId = this.actor.connect('event',
@@ -236,7 +254,7 @@ _Draggable.prototype = {
                                  { scale_x: scale * origScale,
                                    scale_y: scale * origScale,
                                    time: SCALE_ANIMATION_TIME,
-                                   transition: "easeOutQuad",
+                                   transition: 'easeOutQuad',
                                    onUpdate: function() {
                                        let currentScale = this._dragActor.scale_x / origScale;
                                        this._dragOffsetX = currentScale * origDragOffsetX;
@@ -350,7 +368,7 @@ _Draggable.prototype = {
                            scale_y: this._snapBackScale,
                            opacity: this._dragOrigOpacity,
                            time: SNAP_BACK_ANIMATION_TIME,
-                           transition: "easeOutQuad",
+                           transition: 'easeOutQuad',
                            onComplete: this._onSnapBackComplete,
                            onCompleteScope: this,
                            onCompleteParams: [this._dragActor, eventTime]
