@@ -1,6 +1,5 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Big = imports.gi.Big;
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
 const Gdk = imports.gi.Gdk;
@@ -21,25 +20,9 @@ const RedisplayFlags = { NONE: 0,
                          SUBSEARCH: 1 << 2,
                          IMMEDIATE: 1 << 3 };
 
-const ITEM_DISPLAY_DESCRIPTION_COLOR = new Clutter.Color();
-ITEM_DISPLAY_DESCRIPTION_COLOR.from_pixel(0xffffffbb);
-const DISPLAY_CONTROL_SELECTED_COLOR = new Clutter.Color();
-DISPLAY_CONTROL_SELECTED_COLOR.from_pixel(0x112288ff);
-const PREVIEW_BOX_BACKGROUND_COLOR = new Clutter.Color();
-PREVIEW_BOX_BACKGROUND_COLOR.from_pixel(0xADADADf0);
-
-const DEFAULT_PADDING = 4;
-
+// Used by subclasses
 const ITEM_DISPLAY_ICON_SIZE = 48;
-const DEFAULT_COLUMN_GAP = 6;
-
 const PREVIEW_ICON_SIZE = 96;
-const PREVIEW_BOX_PADDING = 6;
-const PREVIEW_BOX_SPACING = DEFAULT_PADDING;
-const PREVIEW_BOX_CORNER_RADIUS = 10;
-// how far relative to the full item width the preview box should be placed
-const PREVIEW_PLACING = 3/4;
-const PREVIEW_DETAILS_MIN_WIDTH = PREVIEW_ICON_SIZE * 2;
 
 /* This is a virtual class that represents a single display item containing
  * a name, a description, and an icon. It allows selecting an item and represents
@@ -51,7 +34,7 @@ function GenericDisplayItem() {
 
 GenericDisplayItem.prototype = {
     _init: function() {
-        this.actor = new St.BoxLayout({ style_class: "generic-display-item",
+        this.actor = new St.BoxLayout({ style_class: 'generic-display-item',
                                          reactive: true });
 
         this.actor._delegate = this;
@@ -105,7 +88,10 @@ GenericDisplayItem.prototype = {
     // Highlights the item by setting a different background color than the default 
     // if isSelected is true, removes the highlighting otherwise.
     markSelected: function(isSelected) {
-        this.actor.set_style_pseudo_class(isSelected ? "selected" : null);
+        if (isSelected)
+            this.actor.add_style_pseudo_class('selected');
+        else
+            this.actor.remove_style_pseudo_class('selected');
     },
 
     /*
@@ -114,11 +100,10 @@ GenericDisplayItem.prototype = {
      */
     createDetailsActor: function() {
 
-        let details = new Big.Box({ orientation: Big.BoxOrientation.VERTICAL,
-                                    spacing: PREVIEW_BOX_SPACING });
+        let details = new St.BoxLayout({ style_class: 'generic-display-container',
+                                         vertical: true });
 
-        let mainDetails = new Big.Box({ orientation: Big.BoxOrientation.HORIZONTAL,
-                                        spacing: PREVIEW_BOX_SPACING });
+        let mainDetails = new St.BoxLayout({ style_class: 'generic-display-container' });
 
         // Inner box with name and description
         let textDetails = new St.BoxLayout({ style_class: 'generic-display-details',
@@ -132,21 +117,19 @@ GenericDisplayItem.prototype = {
 
         this._detailsDescriptions.push(detailsDescription);
 
-        mainDetails.append(textDetails, Big.BoxPackFlags.EXPAND);
+        mainDetails.add(textDetails, { expand: true });
 
         let previewIcon = this._createPreviewIcon();
         let largePreviewIcon = this._createLargePreviewIcon();
 
         if (previewIcon != null && largePreviewIcon == null) {
-            mainDetails.prepend(previewIcon, Big.BoxPackFlags.NONE);
+            mainDetails.insert_actor(previewIcon, 0);
         }
 
-        details.append(mainDetails, Big.BoxPackFlags.NONE);
+        details.add(mainDetails);
 
         if (largePreviewIcon != null) {
-            let largePreview = new Big.Box({ orientation: Big.BoxOrientation.HORIZONTAL });
-            largePreview.append(largePreviewIcon, Big.BoxPackFlags.NONE);
-            details.append(largePreview, Big.BoxPackFlags.NONE);
+            details.add(largePreviewIcon);
         }
    
         return details;
@@ -161,7 +144,7 @@ GenericDisplayItem.prototype = {
 
     // Performes an action associated with launching this item, such as opening a file or an application.
     launch: function() {
-        throw new Error("Not implemented");
+        throw new Error('Not implemented');
     },
 
     //// Protected methods ////
@@ -193,12 +176,12 @@ GenericDisplayItem.prototype = {
         this._icon = this._createIcon();
         this._iconBin.set_child(this._icon);
 
-        this._name = new St.Label({ style_class: "generic-display-item-name",
+        this._name = new St.Label({ style_class: 'generic-display-item-name',
                                      text: nameText });
         this._infoText.add(this._name);
 
-        this._description = new St.Label({ style_class: "generic-display-item-description",
-                                            text: descriptionText ? descriptionText : "" });
+        this._description = new St.Label({ style_class: 'generic-display-item-description',
+                                            text: descriptionText ? descriptionText : '' });
         this._infoText.add(this._description);
     },
 
@@ -225,12 +208,12 @@ GenericDisplayItem.prototype = {
 
     // Returns an icon for the item.
     _createIcon: function() {
-        throw new Error("Not implemented");
+        throw new Error('Not implemented');
     },
 
     // Returns a preview icon for the item.
     _createPreviewIcon: function() {
-        throw new Error("Not implemented");
+        throw new Error('Not implemented');
     }
 
     //// Private methods ////
@@ -373,7 +356,7 @@ GenericDisplay.prototype = {
         // TODO: figure out why this._list.displayedCount is returning a
         // positive number when this._mathedItems.length is 0
         // This can be triggered if a search string is entered for which there are no matches.
-        // log("this._mathedItems.length: " + this._matchedItems.length + " this._list.displayedCount " + this._list.displayedCount);
+        // log('this._mathedItems.length: ' + this._matchedItems.length + ' this._list.displayedCount ' + this._list.displayedCount);
         return this._matchedItemKeys.length > 0;
     },
 
@@ -419,7 +402,7 @@ GenericDisplay.prototype = {
     // and adds it to the list of displayed items, but does not yet display it.
     _addDisplayItem : function(itemId) {
         if (this._displayedItems.hasOwnProperty(itemId)) {
-            log("Tried adding a display item for " + itemId + ", but an item with this item id is already among displayed items.");
+            log('Tried adding a display item for ' + itemId + ', but an item with this item id is already among displayed items.');
             return;
         }
 
@@ -478,7 +461,7 @@ GenericDisplay.prototype = {
     // Return true if there's an active search or other constraint
     // on the list
     _filterActive: function() {
-        return this._search != "";
+        return this._search != '';
     },
 
     // Called when we are resetting state
@@ -601,13 +584,13 @@ GenericDisplay.prototype = {
     // Implementation should return %true if we are up to date, and %false
     // if a full reload occurred.
     _refreshCache: function() {
-        throw new Error("Not implemented");
+        throw new Error('Not implemented');
     },
 
     // Sets the list of the displayed items based on the default sorting order.
     // The default sorting order is specific to each implementing class.
     _setDefaultList: function() {
-        throw new Error("Not implemented");
+        throw new Error('Not implemented');
     },
 
     // Compares items associated with the item ids based on the order in which the
@@ -615,18 +598,18 @@ GenericDisplay.prototype = {
     // Intended to be used as a compareFunction for array.sort().
     // Returns an integer value indicating the result of the comparison.
     _compareItems: function(itemIdA, itemIdB) {
-        throw new Error("Not implemented");
+        throw new Error('Not implemented');
     },
 
     // Checks if the item info can be a match for the search string. 
     // Returns a boolean flag indicating if that's the case.
     _isInfoMatching: function(itemInfo, search) {
-        throw new Error("Not implemented");
+        throw new Error('Not implemented');
     },
 
     // Creates a display item based on itemInfo.
     _createDisplayItem: function(itemInfo) {
-        throw new Error("Not implemented");
+        throw new Error('Not implemented');
     },
 
     //// Private methods ////
