@@ -2,12 +2,8 @@
 
 const Clutter = imports.gi.Clutter;
 const Lang = imports.lang;
-
-const Main = imports.ui.main;
-const Tweener = imports.ui.tweener;
-
-const SHADE_COLOR = new Clutter.Color();
-SHADE_COLOR.from_pixel(0x00000044);
+const Meta = imports.gi.Meta;
+const St = imports.gi.St;
 
 /**
  * Lightbox:
@@ -38,11 +34,10 @@ Lightbox.prototype = {
     _init : function(container, inhibitEvents, width, height) {
         this._container = container;
         this._children = container.get_children();
-        this.actor = new Clutter.Rectangle({ color: SHADE_COLOR,
-                                             x: 0,
-                                             y: 0,
-                                             border_width: 0,
-                                             reactive: inhibitEvents });
+        this.actor = new St.Bin({ x: 0,
+                                  y: 0,
+                                  style_class: 'lightbox',
+                                  reactive: inhibitEvents });
 
         container.add_actor(this.actor);
         this.actor.raise_top();
@@ -66,8 +61,13 @@ Lightbox.prototype = {
     },
 
     _allocationChanged : function(container, box, flags) {
-        this.actor.width = this._container.width;
-        this.actor.height = this._container.height;
+        Meta.later_add(Meta.LaterType.BEFORE_REDRAW, Lang.bind(this, function() {
+            this.actor.width = this.width;
+            this.actor.height = this.height;
+            return false;
+        }));
+        this.width = this._container.width;
+        this.height = this._container.height;
     },
 
     _actorAdded : function(container, newChild) {
