@@ -1,5 +1,11 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
+imports.gi.versions.Clutter = '1.0';
+imports.gi.versions.Gio = '2.0';
+imports.gi.versions.Gdk = '2.0';
+imports.gi.versions.GdkPixbuf = '2.0';
+imports.gi.versions.Gtk = '2.0';
+
 const Clutter = imports.gi.Clutter;
 const DBus = imports.dbus;
 const Gdk = imports.gi.Gdk;
@@ -23,37 +29,41 @@ const RunDialog = imports.ui.runDialog;
 const LookingGlass = imports.ui.lookingGlass;
 const NotificationDaemon = imports.ui.notificationDaemon;
 const WindowAttentionHandler = imports.ui.windowAttentionHandler;
+const Scripting = imports.ui.scripting;
 const ShellDBus = imports.ui.shellDBus;
-const Sidebar = imports.ui.sidebar;
+const TelepathyClient = imports.ui.telepathyClient;
 const WindowManager = imports.ui.windowManager;
+const Magnifier = imports.ui.magnifier;
 
 const DEFAULT_BACKGROUND_COLOR = new Clutter.Color();
 DEFAULT_BACKGROUND_COLOR.from_pixel(0x2266bbff);
 
 let chrome = null;
 let panel = null;
-let sidebar = null;
 let placesManager = null;
 let overview = null;
 let runDialog = null;
 let lookingGlass = null;
 let wm = null;
-let notificationDaemon = null;
 let messageTray = null;
+let notificationDaemon = null;
 let windowAttentionHandler = null;
+let telepathyClient = null;
 let recorder = null;
 let shellDBusService = null;
 let modalCount = 0;
 let modalActorFocusStack = [];
+let uiGroup = null;
+let magnifier = null;
 let _errorLogStack = [];
 let _startDate;
 
 let background = null;
 
 function start() {
-    // Add a binding for "global" in the global JS namespace; (gjs
+    // Add a binding for 'global' in the global JS namespace; (gjs
     // keeps the web browser convention of having that namespace be
-    // called "window".)
+    // called 'window'.)
     window.global = Shell.Global.get();
 
     // Now monkey patch utility functions into the global proxy;
@@ -62,7 +72,7 @@ function start() {
     global.logError = _logError;
     global.log = _logDebug;
 
-    Gio.DesktopAppInfo.set_desktop_env("GNOME");
+    Gio.DesktopAppInfo.set_desktop_env('GNOME');
 
     global.grab_dbus_service();
     shellDBusService = new ShellDBus.GnomeShell();
@@ -90,36 +100,36 @@ function start() {
     // when we are running inside Xephyr.
     global.stage.color = DEFAULT_BACKGROUND_COLOR;
 
-    // Mutter currently hardcodes putting "Yessir. The compositor is running""
-    // in the Overview. Clear that out.
-    let children = global.overlay_group.get_children();
-    for (let i = 0; i < children.length; i++)
-        children[i].destroy();
-
     let themeContext = St.ThemeContext.get_for_stage (global.stage);
-    let stylesheetPath = global.datadir + "/theme/gnome-shell.css";
+    let stylesheetPath = global.datadir + '/theme/gnome-shell.css';
     let theme = new St.Theme ({ application_stylesheet: stylesheetPath });
     themeContext.set_theme (theme);
 
     let shellwm = global.window_manager;
-    shellwm.takeover_keybinding("panel_main_menu");
-    shellwm.connect("keybinding::panel_main_menu", function () {
+    shellwm.takeover_keybinding('panel_main_menu');
+    shellwm.connect('keybinding::panel_main_menu', function () {
         overview.toggle();
     });
-    shellwm.takeover_keybinding("panel_run_dialog");
-    shellwm.connect("keybinding::panel_run_dialog", function () {
+    shellwm.takeover_keybinding('panel_run_dialog');
+    shellwm.connect('keybinding::panel_run_dialog', function () {
        getRunDialog().open();
     });
 
+    // Set up stage hierarchy to group all UI actors under one container.
+    uiGroup = new Clutter.Group();
+    global.window_group.reparent(uiGroup);
+    global.overlay_group.reparent(uiGroup);
+    global.stage.add_actor(uiGroup);
+
     placesManager = new PlaceDisplay.PlacesManager();
     overview = new Overview.Overview();
     chrome = new Chrome.Chrome();
     panel = new Panel.Panel();
-    sidebar = new Sidebar.Sidebar();
     wm = new WindowManager.WindowManager();
+    messageTray = new MessageTray.MessageTray();
     notificationDaemon = new NotificationDaemon.NotificationDaemon();
     windowAttentionHandler = new WindowAttentionHandler.WindowAttentionHandler();
-    messageTray = new MessageTray.MessageTray();
+    telepathyClient = new TelepathyClient.Client();
 
     _startDate = new Date();
 
@@ -133,9 +143,9 @@ function start() {
         } else {
             //read the parameters from GConf always in case they have changed
             let gconf = Shell.GConf.get_default();
-            recorder.set_framerate(gconf.get_int("recorder/framerate"));
-            recorder.set_filename("shell-%d%u-%c." + gconf.get_string("recorder/file_extension"));
-            let pipeline = gconf.get_string("recorder/pipeline");
+            recorder.set_framerate(gconf.get_int('recorder/framerate'));
+            recorder.set_filename('shell-%d%u-%c.' + gconf.get_string('recorder/file_extension'));
+            let pipeline = gconf.get_string('recorder/pipeline');
             if (!pipeline.match(/^\s*$/))
                 recorder.set_pipeline(pipeline);
             else
@@ -161,10 +171,23 @@ function start() {
 
     global.stage.connect('captured-event', _globalKeyPressHandler);
 
+    // Install magnifier.
+    magnifier = new Magnifier.Magnifier();
+
+    // Perform initial relayout here
+    _relayout();
+
     _log('info', 'loaded at ' + _startDate);
     log('GNOME Shell started at ' + _startDate);
 
     Mainloop.idle_add(_removeUnusedWorkspaces);
+
+    let perfModuleName = GLib.getenv("SHELL_PERF_MODULE");
+    if (perfModuleName) {
+        let perfOutput = GLib.getenv("SHELL_PERF_OUTPUT");
+        let module = eval('imports.perf.' + perfModuleName + ';');
+        Scripting.runPerfScript(module, perfOutput);
+    }
 }
 
 /**
@@ -185,7 +208,7 @@ function _log(category, msg) {
         for (let i = 2; i < arguments.length; i++) {
             text += JSON.stringify(arguments[i]);
             if (i < arguments.length - 1)
-                text += " ";
+                text += ' ';
         }
     }
     _errorLogStack.push({timestamp: new Date().getTime(),
@@ -274,8 +297,8 @@ function _globalKeyPressHandler(actor, event) {
         if (symbol == Clutter.Print) {
             // We want to be able to take screenshots of the shell at all times
             let gconf = Shell.GConf.get_default();
-            let command = gconf.get_string("/apps/metacity/keybinding_commands/command_screenshot");
-            if (command != null && command != "") {
+            let command = gconf.get_string('/apps/metacity/keybinding_commands/command_screenshot');
+            if (command != null && command != '') {
                 let [ok, len, args] = GLib.shell_parse_argv(command);
                 let p = new Shell.Process({'args' : args});
                 p.run();
@@ -285,6 +308,9 @@ function _globalKeyPressHandler(actor, event) {
         }
     } else if (type == Clutter.EventType.KEY_RELEASE) {
         let symbol = event.get_key_symbol();
+        let keyCode = event.get_key_code();
+        let modifierState = Shell.get_event_state(event);
+        // Check the overview key first, this isn't a Meta.KeyBindingAction yet
         if (symbol == Clutter.Super_L || symbol == Clutter.Super_R) {
             // The super key is the default for triggering the overview, and should
             // get us out of the overview when we are already in it.
@@ -292,8 +318,25 @@ function _globalKeyPressHandler(actor, event) {
                 overview.hide();
 
             return true;
-        } else if (symbol == Clutter.F2 && (Shell.get_event_state(event) & Clutter.ModifierType.MOD1_MASK)) {
-            getRunDialog().open();
+        }
+
+        // Whitelist some of the Metacity actions
+        let display = global.screen.get_display();
+        let activeWorkspaceIndex = global.screen.get_active_workspace_index();
+
+        // This relies on the fact that Clutter.ModifierType is the same as Gdk.ModifierType
+        let action = display.get_keybinding_action(symbol, keyCode, modifierState);
+        switch (action) {
+            case Meta.KeyBindingAction.WORKSPACE_LEFT:
+                wm.actionMoveWorkspaceLeft();
+                return true;
+            case Meta.KeyBindingAction.WORKSPACE_RIGHT:
+                wm.actionMoveWorkspaceRight();
+                return true;
+            case Meta.KeyBindingAction.PANEL_RUN_DIALOG:
+            case Meta.KeyBindingAction.COMMAND_2:
+                getRunDialog().open();
+                return true;
         }
     }
 
@@ -328,7 +371,7 @@ function _findModal(actor) {
 function pushModal(actor) {
     if (modalCount == 0) {
         if (!global.begin_modal(global.get_current_time())) {
-            log("pushModal: invocation of begin_modal failed");
+            log('pushModal: invocation of begin_modal failed');
             return false;
         }
     }
@@ -471,7 +514,7 @@ function _queueBeforeRedraw(workId) {
         Meta.later_add(Meta.LaterType.BEFORE_REDRAW, function () {
             _runBeforeRedrawQueue();
             return false;
-        }, null);
+        });
     }
 }
 
@@ -495,7 +538,7 @@ function _queueBeforeRedraw(workId) {
  */
 function initializeDeferredWork(actor, callback, props) {
     // Turn into a string so we can use as an object property
-    let workId = "" + (++_deferredWorkSequence);
+    let workId = '' + (++_deferredWorkSequence);
     _deferredWorkData[workId] = { 'actor': actor,
                                   'callback': callback };
     actor.connect('notify::mapped', function () {
@@ -525,7 +568,7 @@ function initializeDeferredWork(actor, callback, props) {
 function queueDeferredWork(workId) {
     let data = _deferredWorkData[workId];
     if (!data) {
-        global.logError("invalid work id ", workId);
+        global.logError('invalid work id ', workId);
         return;
     }
     if (_deferredWorkQueue.indexOf(workId) < 0)
