@@ -1,6 +1,7 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
 const Clutter = imports.gi.Clutter;
+const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Pango = imports.gi.Pango;
@@ -15,7 +16,7 @@ const ANIMATION_TIME = 0.2;
 const NOTIFICATION_TIMEOUT = 4;
 const SUMMARY_TIMEOUT = 1;
 
-const MESSAGE_TRAY_TIMEOUT = 0.2;
+const HIDE_TIMEOUT = 0.2;
 
 const ICON_SIZE = 24;
 
@@ -29,10 +30,10 @@ const State = {
 function _cleanMarkup(text) {
     // Support &amp;, &quot;, &apos;, &lt; and &gt;, escape all other
     // occurrences of '&'.
-    let _text = text.replace(/&(?!amp;|quot;|apos;|lt;|gt;)/g, "&amp;");
+    let _text = text.replace(/&(?!amp;|quot;|apos;|lt;|gt;)/g, '&amp;');
     // Support <b>, <i>, and <u>, escape anything else
     // so it displays as raw markup.
-    return _text.replace(/<(\/?[^biu]>|[^>\/][^>])/g, "&lt;$1");
+    return _text.replace(/<(\/?[^biu]>|[^>\/][^>])/g, '&lt;$1');
 }
 
 // Notification:
@@ -46,11 +47,15 @@ function _cleanMarkup(text) {
 // @source's icon, @title (in bold) and @banner, all on a single line
 // (with @banner ellipsized if necessary).
 //
-// Additional notification details can be added via addBody(),
-// addAction(), and addActor(). If any of these are called, then the
-// notification will expand to show the additional actors (while
-// hiding the @banner) if the pointer is moved into it while it is
-// visible.
+// Additional notification details can be added, in which case the
+// notification can be expanded by moving the pointer into it. In
+// expanded mode, the banner text disappears, and there can be one or
+// more rows of additional content. This content is put inside a
+// scrollview, so if it gets too tall, the notification will scroll
+// rather than continuing to grow. In addition to this main content
+// area, there is also a single-row "action area", which is not
+// scrolled and can contain a single actor. There are also convenience
+// methods for creating a button box in the action area.
 //
 // If @bannerBody is %true, then @banner will also be used as the body
 // of the notification (as with addBody()) when the banner is expanded.
@@ -65,45 +70,52 @@ Notification.prototype = {
         this.id = id;
         this.source = source;
         this._bannerBody = bannerBody;
+        this.urgent = false;
 
         source.connect('clicked', Lang.bind(this,
             function() {
                 this.emit('dismissed');
             }));
 
-        this.actor = new St.Table({ name: 'notification' });
+        this.actor = new St.Table({ name: 'notification',
+                                    reactive: true });
+        this.actor.connect('style-changed', Lang.bind(this, this._styleChanged));
         this.update(title, banner, true);
     },
 
     // update:
     // @title: the new title
     // @banner: the new banner
-    // @clear: whether or not to clear out extra actors
+    // @clear: whether or not to clear out body and action actors
     //
     // Updates the notification by regenerating its icon and updating
     // the title/banner. If @clear is %true, it will also remove any
     // additional actors/action buttons previously added.
     update: function(title, banner, clear) {
-        let children = this.actor.get_children();
-        for (let i = 0; i < children.length; i++) {
-            let meta = this.actor.get_child_meta(children[i]);
-            if (clear || meta.row == 0 || (this._bannerBody && meta.row == 1))
-                children[i].destroy();
+        if (this._icon)
+            this._icon.destroy();
+        if (this._bannerBox)
+            this._bannerBox.destroy();
+        if (this._scrollArea && (this._bannerBody || clear)) {
+            this._scrollArea.destroy();
+            this._scrollArea = null;
+            this._contentArea = null;
         }
-        if (clear) {
-            this.actions = {};
-            this._actionBox = null;
+        if (this._actionArea && clear) {
+            this._actionArea.destroy();
+            this._actionArea = null;
+            this._buttonBox = null;
         }
 
-        let icon = this.source.createIcon(ICON_SIZE);
-        icon.reactive = true;
-        this.actor.add(icon, { row: 0,
-                               col: 0,
-                               x_expand: false,
-                               y_expand: false,
-                               y_fill: false });
+        this._icon = this.source.createIcon(ICON_SIZE);
+        this._icon.reactive = true;
+        this.actor.add(this._icon, { row: 0,
+                                     col: 0,
+                                     x_expand: false,
+                                     y_expand: false,
+                                     y_fill: false });
 
-        icon.connect('button-release-event', Lang.bind(this,
+        this._icon.connect('button-release-event', Lang.bind(this,
             function () {
                 this.source.clicked();
             }));
@@ -136,67 +148,39 @@ Notification.prototype = {
         banner = banner ? _cleanMarkup(banner.replace(/\n/g, '  ')) : '';
         this._bannerLabel.clutter_text.set_markup(banner);
         this._bannerBox.add_actor(this._bannerLabel);
+
+        // Add the bannerBody now if we know for sure we'll need it
+        if (this._bannerBodyText && this._bannerBodyText.indexOf('\n') > -1)
+            this._addBannerBody();
     },
 
     // addActor:
-    // @actor: actor to add to the notification
-    // @props: (optional) child properties
+    // @actor: actor to add to the body of the notification
     //
-    // Adds @actor to the notification's St.Table, using @props.
-    //
-    // If @props does not specify a %row, then @actor will be added
-    // to the bottom of the notification (unless there are action
-    // buttons present, in which case it will be added above them).
-    //
-    // If @props does not specify a %col, it will default to column 1.
-    // (Normally only the icon is in column 0.)
-    //
-    // If @props specifies an already-occupied cell, then the existing
-    // contents of the table will be shifted down to make room for it.
-    addActor: function(actor, props) {
-        if (!props)
-            props = {};
-
-        if (!('col' in props))
-            props.col = 1;
-
-        if ('row' in props) {
-            let children = this.actor.get_children();
-            let i, meta, collision = false;
-
-            for (i = 0; i < children.length; i++) {
-                meta = this.actor.get_child_meta(children[i]);
-                if (meta.row == props.row && meta.col == props.col) {
-                    collision = true;
-                    break;
-                }
-            }
-
-            if (collision) {
-                for (i = 0; i < children.length; i++) {
-                    meta = this.actor.get_child_meta(children[i]);
-                    if (meta.row >= props.row)
-                        meta.row++;
-                }
-            }
-        } else {
-            if (this._actionBox) {
-                props.row = this.actor.row_count - 1;
-                this.actor.get_child_meta(this._actionBox).row++;
-            } else {
-                props.row = this.actor.row_count;
-            }
+    // Appends @actor to the notification's body
+    addActor: function(actor) {
+        if (!this._scrollArea) {
+            this._scrollArea = new St.ScrollView({ name: 'notification-scrollview',
+                                                   vscrollbar_policy: Gtk.PolicyType.AUTOMATIC,
+                                                   hscrollbar_policy: Gtk.PolicyType.NEVER,
+                                                   vshadows: true });
+            this.actor.add(this._scrollArea, { row: 1,
+                                               col: 1 });
+            this._contentArea = new St.BoxLayout({ name: 'notification-body',
+                                                   vertical: true });
+            this._scrollArea.add_actor(this._contentArea);
         }
 
-        this.actor.add(actor, props);
+        this._contentArea.add(actor);
     },
 
     // addBody:
     // @text: the text
-    // @props: (optional) properties for addActor()
     //
     // Adds a multi-line label containing @text to the notification.
-    addBody: function(text, props) {
+    //
+    // Return value: the newly-added label
+    addBody: function(text) {
         let body = new St.Label();
         body.clutter_text.line_wrap = true;
         body.clutter_text.line_wrap_mode = Pango.WrapMode.WORD_CHAR;
@@ -205,15 +189,56 @@ Notification.prototype = {
         text = text ? _cleanMarkup(text) : '';
         body.clutter_text.set_markup(text);
 
-        this.addActor(body, props);
+        this.addActor(body);
+        return body;
     },
 
     _addBannerBody: function() {
-        this.addBody(this._bannerBodyText, { row: 1 });
+        this.addBody(this._bannerBodyText);
         this._bannerBodyText = null;
     },
 
-    // addAction:
+    // scrollTo:
+    // @side: St.Side.TOP or St.Side.BOTTOM
+    //
+    // Scrolls the content area (if scrollable) to the indicated edge
+    scrollTo: function(side) {
+        // Hack to force a relayout, since the caller probably
+        // just added or removed something to scrollArea, and
+        // the adjustment needs to reflect that.
+        global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, 0, 0);
+
+        let adjustment = this._scrollArea.vscroll.adjustment;
+        if (side == St.Side.TOP)
+            adjustment.value = adjustment.lower;
+        else if (side == St.Side.BOTTOM)
+            adjustment.value = adjustment.upper;
+    },
+
+    // setActionArea:
+    // @actor: the actor
+    // @props: (option) St.Table child properties
+    //
+    // Puts @actor into the action area of the notification, replacing
+    // the previous contents
+    setActionArea: function(actor, props) {
+        if (this._actionArea) {
+            this._actionArea.destroy();
+            this._actionArea = null;
+            if (this._buttonBox)
+                this._buttonBox = null;
+        }
+        this._actionArea = actor;
+
+        if (!props)
+            props = {};
+        props.row = 2;
+        props.col = 1;
+
+        this.actor.add(this._actionArea, props);
+    },
+
+    // addButton:
     // @id: the action ID
     // @label: the label for the action's button
     //
@@ -223,31 +248,51 @@ Notification.prototype = {
     //
     // If the button is clicked, the notification will emit the
     // %action-invoked signal with @id as a parameter
-    addAction: function(id, label) {
-        if (!this._actionBox) {
+    addButton: function(id, label) {
+        if (!this._buttonBox) {
             if (this._bannerBodyText)
                 this._addBannerBody();
 
             let box = new St.BoxLayout({ name: 'notification-actions' });
-            this.addActor(box, { x_expand: false,
-                                 x_fill: false,
-                                 x_align: St.Align.END });
-            this._actionBox = box;
+            this.setActionArea(box, { x_expand: false,
+                                      x_fill: false,
+                                      x_align: St.Align.END });
+            this._buttonBox = box;
         }
 
         let button = new St.Button({ style_class: 'notification-button',
                                      label: label });
-        this._actionBox.add(button);
+        this._buttonBox.add(button);
         button.connect('clicked', Lang.bind(this, function() { this.emit('action-invoked', id); }));
     },
 
+    setUrgent: function(urgent) {
+        this.urgent = urgent;
+    },
+
+    _styleChanged: function() {
+        let [has_spacing, spacing] = this.actor.get_theme_node().get_length('spacing-columns', false);
+        this._spacing = has_spacing ? spacing : 0;
+
+        // Figure out now (before allocation starts) whether or not we
+        // need to be expandable, and add the expansion row if so
+        if (this._bannerBodyText) {
+            let [minBannerWidth, natBannerWidth] =
+                this._bannerBox.get_preferred_width(-1);
+            let [minNotificationWidth, natNotificationWidth] =
+                this.actor.get_preferred_width(-1);
+
+            if (natBannerWidth > natNotificationWidth)
+                this._addBannerBody();
+        }
+    },
+
     _bannerBoxGetPreferredWidth: function(actor, forHeight, alloc) {
         let [titleMin, titleNat] = this._titleLabel.get_preferred_width(forHeight);
         let [bannerMin, bannerNat] = this._bannerLabel.get_preferred_width(forHeight);
-        let [has_spacing, spacing] = this.actor.get_theme_node().get_length('spacing-columns', false);
 
         alloc.min_size = titleMin;
-        alloc.natural_size = titleNat + (has_spacing ? spacing : 0) + bannerNat;
+        alloc.natural_size = titleNat + this._spacing + bannerNat;
     },
 
     _bannerBoxGetPreferredHeight: function(actor, forWidth, alloc) {
@@ -259,9 +304,6 @@ Notification.prototype = {
         let [titleMinW, titleNatW] = this._titleLabel.get_preferred_width(-1);
         let [titleMinH, titleNatH] = this._titleLabel.get_preferred_height(-1);
         let [bannerMinW, bannerNatW] = this._bannerLabel.get_preferred_width(-1);
-        let [has_spacing, spacing] = this.actor.get_theme_node().get_length('spacing-columns', false);
-        if (!has_spacing)
-            spacing = 0;
         let availWidth = box.x2 - box.x1;
 
         let titleBox = new Clutter.ActorBox();
@@ -270,26 +312,17 @@ Notification.prototype = {
         titleBox.y2 = titleNatH;
         this._titleLabel.allocate(titleBox, flags);
 
-        let overflow = false;
-        if (titleBox.x2 + spacing > availWidth) {
+        if (titleBox.x2 + this._spacing > availWidth) {
             this._bannerLabel.hide();
-            overflow = true;
         } else {
             let bannerBox = new Clutter.ActorBox();
-            bannerBox.x1 = titleBox.x2 + spacing;
+            bannerBox.x1 = titleBox.x2 + this._spacing;
             bannerBox.y1 = 0;
             bannerBox.x2 = Math.min(bannerBox.x1 + bannerNatW, availWidth);
             bannerBox.y2 = titleNatH;
             this._bannerLabel.show();
             this._bannerLabel.allocate(bannerBox, flags);
-
-            if (bannerBox.x2 < bannerBox.x1 + bannerNatW)
-                overflow = true;
         }
-
-        if (this._bannerBodyText &&
-            (overflow || this._bannerBodyText.indexOf('\n') > -1))
-            this._addBannerBody();
     },
 
     popOut: function() {
@@ -299,7 +332,7 @@ Notification.prototype = {
         Tweener.addTween(this._bannerLabel,
                          { opacity: 0,
                            time: ANIMATION_TIME,
-                           transition: "easeOutQuad" });
+                           transition: 'easeOutQuad' });
         return true;
     },
 
@@ -309,7 +342,7 @@ Notification.prototype = {
         Tweener.addTween(this._bannerLabel,
                          { opacity: 255,
                            time: ANIMATION_TIME,
-                           transition: "easeOutQuad" });
+                           transition: 'easeOutQuad' });
         return true;
     },
 
@@ -329,7 +362,6 @@ Source.prototype = {
         this.text = null;
         if (createIcon)
             this.createIcon = createIcon;
-        this.handleReplacing = true;
     },
 
     // This can be overridden by a subclass, or by the createIcon
@@ -339,6 +371,19 @@ Source.prototype = {
     },
 
     notify: function(notification) {
+        if (this.notification)
+            this.notification.disconnect(this._notificationDestroyedId);
+
+        this.notification = notification;
+
+        this._notificationDestroyedId = notification.connect('destroy', Lang.bind(this,
+            function () {
+                if (this.notification == notification) {
+                    this.notification = null;
+                    this._notificationDestroyedId = 0;
+                }
+            }));
+
         this.emit('notify', notification);
     },
 
@@ -359,30 +404,35 @@ function MessageTray() {
 MessageTray.prototype = {
     _init: function() {
         this.actor = new St.BoxLayout({ name: 'message-tray',
-                                        reactive: true });
+                                        reactive: true,
+                                        track_hover: true });
+        this.actor.connect('notify::hover', Lang.bind(this, this._onTrayHoverChanged));
 
-        this._notificationBin = new St.Bin({ reactive: true,
-                                             x_align: St.Align.MIDDLE });
+        this._notificationBin = new St.Bin();
         this.actor.add(this._notificationBin);
         this._notificationBin.hide();
         this._notificationQueue = [];
         this._notification = null;
 
-        this._summaryBin = new St.BoxLayout();
+        this._summaryBin = new St.Bin({ anchor_gravity: Clutter.Gravity.NORTH_EAST });
         this.actor.add(this._summaryBin);
         this._summary = new St.BoxLayout({ name: 'summary-mode',
-                                           reactive: true });
-        this._summaryBin.add(this._summary, { x_align: St.Align.END,
-                                              x_fill: false,
-                                              expand: true });
-        this._summary.connect('enter-event',
-                              Lang.bind(this, this._onSummaryEntered));
-        this._summary.connect('leave-event',
-                              Lang.bind(this, this._onSummaryLeft));
+                                           reactive: true,
+                                           track_hover: true });
+        this._summary.connect('notify::hover', Lang.bind(this, this._onSummaryHoverChanged));
+        this._summaryBin.child = this._summary;
         this._summaryBin.opacity = 0;
 
-        this.actor.connect('enter-event', Lang.bind(this, this._onTrayEntered));
-        this.actor.connect('leave-event', Lang.bind(this, this._onTrayLeft));
+        this._summaryNotificationBin = new St.Bin({ name: 'summary-notification-bin',
+                                                    anchor_gravity: Clutter.Gravity.NORTH_EAST,
+                                                    reactive: true,
+                                                    track_hover: true });
+        this.actor.add(this._summaryNotificationBin);
+        this._summaryNotificationBin.lower_bottom();
+        this._summaryNotificationBin.hide();
+        this._summaryNotificationBin.connect('notify::hover', Lang.bind(this, this._onSummaryNotificationHoverChanged));
+        this._summaryNotification = null;
+        this._hoverSource = null;
 
         this._trayState = State.HIDDEN;
         this._trayLeftTimeoutId = 0;
@@ -392,13 +442,15 @@ MessageTray.prototype = {
         this._pointerInSummary = false;
         this._notificationState = State.HIDDEN;
         this._notificationTimeoutId = 0;
+        this._summaryNotificationState = State.HIDDEN;
+        this._summaryNotificationTimeoutId = 0;
         this._overviewVisible = false;
         this._notificationRemoved = false;
 
-        this.actor.show();
         Main.chrome.addActor(this.actor, { affectsStruts: false,
                                            visibleInOverview: true });
-        Main.chrome.trackActor(this._notificationBin, { affectsStruts: false });
+        Main.chrome.trackActor(this._notificationBin);
+        Main.chrome.trackActor(this._summaryNotificationBin);
 
         global.connect('screen-size-changed',
                        Lang.bind(this, this._setSizePosition));
@@ -424,9 +476,12 @@ MessageTray.prototype = {
         this.actor.x = primary.x;
         this.actor.y = primary.y + primary.height - 1;
         this.actor.width = primary.width;
+        this._notificationBin.x = 0;
+        this._notificationBin.width = primary.width;
 
-        this._notificationBin.x = this._summaryBin.x = 0;
-        this._notificationBin.width = this._summaryBin.width = primary.width;
+        // These work because of their anchor_gravity
+        this._summaryBin.x = primary.width;
+        this._summaryNotificationBin.x = primary.width;
     },
 
     contains: function(source) {
@@ -439,7 +494,8 @@ MessageTray.prototype = {
             return;
         }
 
-        let iconBox = new St.Bin({ reactive: true });
+        let iconBox = new St.Clickable({ style_class: 'summary-icon',
+                                         reactive: true });
         iconBox.child = source.createIcon(ICON_SIZE);
         this._summary.insert_actor(iconBox, 0);
         this._summaryNeedsToBeShown = true;
@@ -448,7 +504,11 @@ MessageTray.prototype = {
 
         source.connect('notify', Lang.bind(this, this._onNotify));
 
-        iconBox.connect('button-release-event', Lang.bind(this,
+        iconBox.connect('notify::hover', Lang.bind(this,
+            function () {
+                this._onSourceHoverChanged(source, iconBox.hover);
+            }));
+        iconBox.connect('clicked', Lang.bind(this,
             function () {
                 source.clicked();
             }));
@@ -479,14 +539,23 @@ MessageTray.prototype = {
         delete this._icons[source.id];
         delete this._sources[source.id];
 
+        let needUpdate = false;
+
         if (this._notification && this._notification.source == source) {
             if (this._notificationTimeoutId) {
                 Mainloop.source_remove(this._notificationTimeoutId);
                 this._notificationTimeoutId = 0;
             }
             this._notificationRemoved = true;
-            this._updateState();
+            needUpdate = true;
         }
+        if (this._hoverSource == source) {
+            this._hoverSource = null;
+            needUpdate = true;
+        }
+
+        if (needUpdate);
+            this._updateState();
     },
 
     removeSourceByApp: function(app) {
@@ -527,42 +596,90 @@ MessageTray.prototype = {
         return null;
     },
 
+    lock: function() {
+        this._locked = true;
+    },
+
+    unlock: function() {
+        this._locked = false;
+
+        this.actor.sync_hover();
+        this._summary.sync_hover();
+
+        this._updateState();
+    },
+
     _onNotify: function(source, notification) {
-        if (!notification.source.handleReplacing || this._getNotification(notification.id, source) == null) {
+        if (notification == this._summaryNotification)
+            return;
+
+        if (this._getNotification(notification.id, source) == null) {
             notification.connect('destroy',
                                  Lang.bind(this, this.removeNotification));
-            this._notificationQueue.push(notification);
+
+            if (notification.urgent)
+                this._notificationQueue.unshift(notification);
+            else
+                this._notificationQueue.push(notification);
         }
 
         this._updateState();
     },
 
-    _onSummaryEntered: function() {
-        this._pointerInSummary = true;
-        this._updateState();
-    },
-
-    _onSummaryLeft: function() {
-        this._pointerInSummary = false;
-        this._updateState();
-    },
-
-    _onTrayEntered: function() {
-        if (this._trayLeftTimeoutId) {
-            Mainloop.source_remove(this._trayLeftTimeoutId);
-            this._trayLeftTimeoutId = 0;
+    _onSourceHoverChanged: function(source, hover) {
+        if (!source.notification)
             return;
+
+        if (this._summaryNotificationTimeoutId != 0) {
+            Mainloop.source_remove(this._summaryNotificationTimeoutId);
+            this._summaryNotificationTimeoutId = 0;
         }
 
-        this._pointerInTray = true;
+        if (hover) {
+            this._hoverSource = source;
+            this._updateState();
+        } else if (this._hoverSource == source) {
+            let timeout = HIDE_TIMEOUT * 1000;
+            this._summaryNotificationTimeoutId = Mainloop.timeout_add(timeout, Lang.bind(this, this._onSourceHoverChangedTimeout, source));
+        }
+    },
+
+    _onSourceHoverChangedTimeout: function(source) {
+        this._summaryNotificationTimeoutId = 0;
+        if (this._hoverSource == source) {
+            this._hoverSource = null;
+            this._updateState();
+        }
+    },
+
+    _onSummaryNotificationHoverChanged: function() {
+        if (!this._summaryNotification)
+            return;
+        this._onSourceHoverChanged(this._summaryNotification.source,
+                                   this._summaryNotificationBin.hover);
+    },
+
+    _onSummaryHoverChanged: function() {
+        this._pointerInSummary = this._summary.hover;
         this._updateState();
     },
 
-    _onTrayLeft: function() {
-        // We wait just a little before hiding the message tray in case the
-        // user quickly moves the mouse back into it.
-        let timeout = MESSAGE_TRAY_TIMEOUT * 1000;
-        this._trayLeftTimeoutId = Mainloop.timeout_add(timeout, Lang.bind(this, this._onTrayLeftTimeout));
+    _onTrayHoverChanged: function() {
+        if (this.actor.hover) {
+            if (this._trayLeftTimeoutId) {
+                Mainloop.source_remove(this._trayLeftTimeoutId);
+                this._trayLeftTimeoutId = 0;
+                return;
+            }
+
+            this._pointerInTray = true;
+            this._updateState();
+        } else {
+            // We wait just a little before hiding the message tray in case the
+            // user quickly moves the mouse back into it.
+            let timeout = HIDE_TIMEOUT * 1000;
+            this._trayLeftTimeoutId = Mainloop.timeout_add(timeout, Lang.bind(this, this._onTrayLeftTimeout));
+        }
     },
 
     _onTrayLeftTimeout: function() {
@@ -575,7 +692,7 @@ MessageTray.prototype = {
 
     // All of the logic for what happens when occurs here; the various
     // event handlers merely update variables such as
-    // "this._pointerInTray", "this._summaryState", etc, and
+    // 'this._pointerInTray', 'this._summaryState', etc, and
     // _updateState() figures out what (if anything) needs to be done
     // at the present time.
     _updateState: function() {
@@ -583,7 +700,7 @@ MessageTray.prototype = {
         let notificationsPending = this._notificationQueue.length > 0;
         let notificationPinned = this._pointerInTray && !this._pointerInSummary && !this._notificationRemoved;
         let notificationExpanded = this._notificationBin.y < 0;
-        let notificationExpired = (this._notificationTimeoutId == 0 && !this._pointerInTray) || this._notificationRemoved;
+        let notificationExpired = (this._notificationTimeoutId == 0 && !this._pointerInTray && !this._locked) || this._notificationRemoved;
 
         if (this._notificationState == State.HIDDEN) {
             if (notificationsPending)
@@ -613,6 +730,22 @@ MessageTray.prototype = {
                 this._hideSummary();
         }
 
+        // Summary notification
+        let haveSummaryNotification = this._hoverSource != null;
+        let summaryNotificationIsMainNotification = (haveSummaryNotification &&
+                                                     this._hoverSource.notification == this._notification);
+        let canShowSummaryNotification = this._summaryState == State.SHOWN;
+        let wrongSummaryNotification = (haveSummaryNotification &&
+                                        this._summaryNotification != this._hoverSource.notification);
+
+        if (this._summaryNotificationState == State.HIDDEN) {
+            if (haveSummaryNotification && !summaryNotificationIsMainNotification && canShowSummaryNotification)
+                this._showSummaryNotification();
+        } else if (this._summaryNotificationState == State.SHOWN) {
+            if (!haveSummaryNotification || !canShowSummaryNotification || wrongSummaryNotification)
+                this._hideSummaryNotification();
+        }
+
         // Tray itself
         let trayIsVisible = (this._trayState == State.SHOWING ||
                              this._trayState == State.SHOWN);
@@ -649,19 +782,19 @@ MessageTray.prototype = {
 
     _showTray: function() {
         let primary = global.get_primary_monitor();
-        this._tween(this.actor, "_trayState", State.SHOWN,
+        this._tween(this.actor, '_trayState', State.SHOWN,
                     { y: primary.y + primary.height - this.actor.height,
                       time: ANIMATION_TIME,
-                      transition: "easeOutQuad"
+                      transition: 'easeOutQuad'
                     });
     },
 
     _hideTray: function() {
         let primary = global.get_primary_monitor();
-        this._tween(this.actor, "_trayState", State.HIDDEN,
+        this._tween(this.actor, '_trayState', State.HIDDEN,
                     { y: primary.y + primary.height - 1,
                       time: ANIMATION_TIME,
-                      transition: "easeOutQuad"
+                      transition: 'easeOutQuad'
                     });
     },
 
@@ -673,14 +806,23 @@ MessageTray.prototype = {
         this._notificationBin.y = this.actor.height;
         this._notificationBin.show();
 
-        this._tween(this._notificationBin, "_notificationState", State.SHOWN,
+        this._tween(this._notificationBin, '_notificationState', State.SHOWN,
                     { y: 0,
                       opacity: 255,
                       time: ANIMATION_TIME,
-                      transition: "easeOutQuad",
+                      transition: 'easeOutQuad',
                       onComplete: this._showNotificationCompleted,
                       onCompleteScope: this
                     });
+
+        if (this._notification.urgent) {
+            // This will overwrite the y tween, but leave the opacity
+            // tween, and so the onComplete will remain as well.
+            this._expandNotification();
+        }
+
+        let [x, y, mods] = global.get_pointer();
+        this._lastSeenMouseY = y;
     },
 
     _showNotificationCompleted: function() {
@@ -690,19 +832,37 @@ MessageTray.prototype = {
     },
 
     _notificationTimeout: function() {
-        this._notificationTimeoutId = 0;
-        this._updateState();
+        let [x, y, mods] = global.get_pointer();
+        if (y > this._lastSeenMouseY + 10 && y < this.actor.y) {
+            // The mouse is moving towards the notification, so don't
+            // hide it yet. (We just create a new timeout (and destroy
+            // the old one) each time because the bookkeeping is
+            // simpler.)
+            this._lastSeenMouseY = y;
+            this._notificationTimeoutId =
+                Mainloop.timeout_add(1000,
+                                     Lang.bind(this, this._notificationTimeout));
+        } else {
+            this._notificationTimeoutId = 0;
+            this._updateState();
+        }
+
         return false;
     },
 
     _hideNotification: function() {
         this._notification.popIn();
 
-        this._tween(this._notificationBin, "_notificationState", State.HIDDEN,
+        if (this._reExpandNotificationId) {
+            this._notificationBin.disconnect(this._reExpandNotificationId);
+            this._reExpandNotificationId = 0;
+        }
+
+        this._tween(this._notificationBin, '_notificationState', State.HIDDEN,
                     { y: this.actor.height,
                       opacity: 0,
                       time: ANIMATION_TIME,
-                      transition: "easeOutQuad",
+                      transition: 'easeOutQuad',
                       onComplete: this._hideNotificationCompleted,
                       onCompleteScope: this
                     });
@@ -717,11 +877,14 @@ MessageTray.prototype = {
 
     _expandNotification: function() {
         if (this._notification && this._notification.popOut()) {
-            this._tween(this._notificationBin, "_notificationState", State.SHOWN,
+            this._tween(this._notificationBin, '_notificationState', State.SHOWN,
                         { y: this.actor.height - this._notificationBin.height,
                           time: ANIMATION_TIME,
-                          transition: "easeOutQuad"
+                          transition: 'easeOutQuad'
                         });
+
+            if (!this._reExpandNotificationId)
+                this._reExpandNotificationId = this._notificationBin.connect('notify::height', Lang.bind(this, this._expandNotification));
         }
     },
 
@@ -729,11 +892,11 @@ MessageTray.prototype = {
         let primary = global.get_primary_monitor();
         this._summaryBin.opacity = 0;
         this._summaryBin.y = this.actor.height;
-        this._tween(this._summaryBin, "_summaryState", State.SHOWN,
+        this._tween(this._summaryBin, '_summaryState', State.SHOWN,
                     { y: 0,
                       opacity: 255,
                       time: ANIMATION_TIME,
-                      transition: "easeOutQuad",
+                      transition: 'easeOutQuad',
                       onComplete: this._showSummaryCompleted,
                       onCompleteScope: this,
                       onCompleteParams: [withTimeout]
@@ -757,11 +920,68 @@ MessageTray.prototype = {
     },
 
     _hideSummary: function() {
-        this._tween(this._summaryBin, "_summaryState", State.HIDDEN,
+        this._tween(this._summaryBin, '_summaryState', State.HIDDEN,
                     { opacity: 0,
                       time: ANIMATION_TIME,
-                      transition: "easeOutQuad"
+                      transition: 'easeOutQuad'
                     });
         this._summaryNeedsToBeShown = false;
+    },
+
+    _showSummaryNotification: function() {
+        this._summaryNotification = this._hoverSource.notification;
+
+        let index = this._notificationQueue.indexOf(this._summaryNotification);
+        if (index != -1)
+            this._notificationQueue.splice(index, 1);
+
+        this._summaryNotificationBin.child = this._summaryNotification.actor;
+        this._summaryNotification.popOut();
+
+        this._summaryNotificationBin.opacity = 0;
+        this._summaryNotificationBin.y = this.actor.height;
+        this._summaryNotificationBin.show();
+
+        this._tween(this._summaryNotificationBin, '_summaryNotificationState', State.SHOWN,
+                    { y: this.actor.height - this._summaryNotificationBin.height,
+                      opacity: 255,
+                      time: ANIMATION_TIME,
+                      transition: 'easeOutQuad'
+                    });
+
+        if (!this._reExpandSummaryNotificationId)
+            this._reExpandSummaryNotificationId = this._summaryNotificationBin.connect('notify::height', Lang.bind(this, this._reExpandSummaryNotification));
+    },
+
+    _reExpandSummaryNotification: function() {
+        this._tween(this._summaryNotificationBin, '_summaryNotificationState', State.SHOWN,
+                    { y: this.actor.height - this._summaryNotificationBin.height,
+                      time: ANIMATION_TIME,
+                      transition: 'easeOutQuad'
+                    });
+    },
+
+    _hideSummaryNotification: function() {
+        this._summaryNotification.popIn();
+
+        this._tween(this._summaryNotificationBin, '_summaryNotificationState', State.HIDDEN,
+                    { y: this.actor.height,
+                      opacity: 0,
+                      time: ANIMATION_TIME,
+                      transition: 'easeOutQuad',
+                      onComplete: this._hideSummaryNotificationCompleted,
+                      onCompleteScope: this
+                    });
+
+        if (this._reExpandSummaryNotificationId) {
+            this._summaryNotificationBin.disconnect(this._reExpandSummaryNotificationId);
+            this._reExpandSummaryNotificationId = 0;
+        }
+    },
+
+    _hideSummaryNotificationCompleted: function() {
+        this._summaryNotificationBin.hide();
+        this._summaryNotificationBin.child = null;
+        this._summaryNotification = null;
     }
 };
