@@ -5,13 +5,12 @@ const GLib = imports.gi.GLib;
 const Lang = imports.lang;
 const Shell = imports.gi.Shell;
 const Mainloop = imports.mainloop;
+const St = imports.gi.St;
 
 const Main = imports.ui.main;
 const MessageTray = imports.ui.messageTray;
 const Params = imports.misc.params;
 
-const EMPATHY = 'Empathy';
-
 let nextNotificationId = 1;
 
 // Should really be defined in dbus.js
@@ -20,17 +19,17 @@ const BusIface = {
     methods: [{ name: 'GetConnectionUnixProcessID',
                 inSignature: 's',
                 outSignature: 'i' }]
-}
+};
 
 const Bus = function () {
     this._init();
-}
+};
 
 Bus.prototype = {
      _init: function() {
          DBus.session.proxifyObject(this, 'org.freedesktop.DBus', '/org/freedesktop/DBus');
      }
-}
+};
 
 DBus.proxifyPrototype(Bus.prototype, BusIface);
 
@@ -100,6 +99,11 @@ NotificationDaemon.prototype = {
                                   Lang.bind(this, this._lostName));
 
         this._currentNotifications = {};
+
+        Shell.WindowTracker.get_default().connect('notify::focus-app',
+            Lang.bind(this, this._onFocusAppChanged));
+        Main.overview.connect('hidden',
+            Lang.bind(this, this._onFocusAppChanged));
     },
 
     _acquiredName: function() {
@@ -117,10 +121,10 @@ NotificationDaemon.prototype = {
             // kill the notification-daemon. pkill is more portable
             // than killall, but on Linux at least it won't match if
             // you pass more than 15 characters of the process name...
-            // However, if you use the "-f" flag to match the entire
+            // However, if you use the '-f' flag to match the entire
             // command line, it will work, but we have to be careful
-            // in that case that we don't match "gedit
-            // notification-daemon.c" or whatever...
+            // in that case that we don't match 'gedit
+            // notification-daemon.c' or whatever...
             let p = new Shell.Process({ args: ['pkill', '-f',
                                                '^([^ ]*/)?(notification-daemon|notify-osd)$']});
             p.run();
@@ -133,46 +137,27 @@ NotificationDaemon.prototype = {
 
     Notify: function(appName, replacesId, icon, summary, body,
                      actions, hints, timeout) {
-        let notification, id;
-        // We associate each application with a source and set the source id to be based on
-        // the appName. We support application updates by creating a new Notification object
-        // only if replacesId was not specified or if we no longer have the notification with
-        // the specified replacesId.
-        //
-        // We are planning to add Empathy-specific features in the message tray, but in the
-        // meantime we handle Empathy notifications received through the notification daemon
-        // differently from other notifications.
-        // 1) We display different people sending messages as different sources. So we use
-        //    notification id instead of the appName when creating id for the source.
-        // 2) We queue notification with different messages to show them one after another,
-        //    rather than replace the notification on the spot. So we create a new Notification
-        //    object each time, disregarding the fact that we might already have another
-        //    notification with the same replacesId.
-        // Empathy uses replacesId for all the notifications from the same window until the
-        // notification with that id is dismissed. Notifications from different people in
-        // different tabs have the same replacesId. So while being closer to the eventual design,
-        // our special-handling of Empathy notifications is somewhat buggy. The user might end up
-        // with multiple icons for the same person if the user dismisses the window for that person
-        // (which doesn't result in removing the icon) or the user might not get the icon for
-        // the person after a new notification in one of the tabs of the chat window for which
-        // another person's icon is already displayed.
+        let source = Main.messageTray.getSource(this._sourceId(appName));
+        let id = null;
 
-        let isEmpathy = appName == EMPATHY;
-        if (replacesId != 0) {
-            id = replacesId;
-            if (!isEmpathy)
-                notification = this._currentNotifications[id];
-        } else {
+        // Filter out notifications from Empathy, since we
+        // handle that information from telepathyClient.js
+        if (appName == 'Empathy') {
             id = nextNotificationId++;
+            Mainloop.idle_add(Lang.bind(this,
+                                        function () {
+                                            this._emitNotificationClosed(id, NotificationClosedReason.DISMISSED);
+                                        }));
+            return id;
         }
 
-        let sourceId = this._sourceId(isEmpathy ? id : appName);
-        let source = Main.messageTray.getSource(sourceId);
+        hints = Params.parse(hints, { urgency: Urgency.NORMAL }, true);
 
-        // Source may be null if we have never received a notification from
-        // this app or if all notifications from this app have been acknowledged.
+        // Source may be null if we have never received a notification
+        // from this app or if all notifications from this app have
+        // been acknowledged.
         if (source == null) {
-            source = new Source(sourceId, icon, hints);
+            source = new Source(this._sourceId(appName), icon, hints);
             Main.messageTray.add(source);
 
             source.connect('clicked', Lang.bind(this,
@@ -184,7 +169,8 @@ NotificationDaemon.prototype = {
             let busProxy = new Bus();
             busProxy.GetConnectionUnixProcessIDRemote(sender, function (result, excp) {
                 let app = Shell.WindowTracker.get_default().get_app_from_pid(result);
-                source.setApp(app);
+                if (app)
+                    source.setApp(app);
             });
         } else {
             source.update(icon, hints);
@@ -201,14 +187,15 @@ NotificationDaemon.prototype = {
             }
         }
 
+        let notification;
+        if (replacesId != 0) {
+            id = replacesId;
+            notification = this._currentNotifications[id];
+        }
+
         if (notification == null) {
+            id = nextNotificationId++;
             notification = new MessageTray.Notification(id, source, summary, body, true);
-            // This will result in us keeping only the latest Empathy notification with the given
-            // id in this._currentNotifications, which will only affect not being able to close all
-            // the Empathy notifications with a given id in CloseNotification(). Since this not a
-            // a likely scenario and this special-casing of Empathy in the notification daemon is
-            // temporary, it doesn't seem worthwhile to change this._currentNotifications to
-            // {id, [array of notifications]} just for that case.
             this._currentNotifications[id] = notification;
             notification.connect('dismissed', Lang.bind(this,
                 function(n) {
@@ -223,10 +210,12 @@ NotificationDaemon.prototype = {
 
         if (actions.length) {
             for (let i = 0; i < actions.length - 1; i += 2)
-                notification.addAction(actions[i], actions[i + 1]);
+                notification.addButton(actions[i], actions[i + 1]);
             notification.connect('action-invoked', Lang.bind(this, this._actionInvoked, source, id));
         }
 
+        notification.setUrgent(hints.urgency == Urgency.CRITICAL);
+
         source.notify(notification);
         return id;
     },
@@ -260,6 +249,12 @@ NotificationDaemon.prototype = {
         ];
     },
 
+    _onFocusAppChanged: function() {
+        let tracker = Shell.WindowTracker.get_default();
+        if (tracker.focus_app)
+            Main.messageTray.removeSourceByApp(tracker.focus_app);
+    },
+
     _actionInvoked: function(notification, action, source, id) {
         this._emitActionInvoked(id, action);
         source.destroy();
@@ -293,22 +288,20 @@ Source.prototype = {
     _init: function(sourceId, icon, hints) {
         MessageTray.Source.prototype._init.call(this, sourceId);
 
+        this.app = null;
+        this._openAppRequested = false;
+
         this.update(icon, hints);
     },
 
     update: function(icon, hints) {
-        hints = Params.parse(hints, { urgency: Urgency.NORMAL }, true);
-
         this._icon = icon;
         this._iconData = hints.icon_data;
         this._urgency = hints.urgency;
-
-        this.app = null;
-        this._openAppRequested = false;
     },
 
     createIcon: function(size) {
-        let textureCache = Shell.TextureCache.get_default();
+        let textureCache = St.TextureCache.get_default();
 
         if (this._icon) {
             if (this._icon.substr(0, 7) == 'file://')
@@ -347,9 +340,6 @@ Source.prototype = {
         this.app = app;
         if (this._openAppRequested)
             this.openApp();
-
-        if (app.get_name() == EMPATHY)
-            this.handleReplacing = false;
     },
 
     openApp: function() {
