@@ -1,6 +1,5 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Big = imports.gi.Big;
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
@@ -73,7 +72,7 @@ const SHADOW_WIDTH = 6;
 
 const NUMBER_OF_SECTIONS_IN_SEARCH = 2;
 
-const INFO_BAR_HIDE_TIMEOUT = 30;
+const INFO_BAR_HIDE_TIMEOUT = 10;
 
 let wideScreen = false;
 let displayGridColumnWidth = null;
@@ -108,9 +107,6 @@ InfoBar.prototype = {
 
         this._undoCallback = null;
         this._undo.connect('clicked', Lang.bind(this, this._onUndoClicked));
-
-        this._overviewWasHidden = false;
-        this._hidingOverviewId = 0;
     },
 
     _onUndoClicked: function() {
@@ -128,7 +124,6 @@ InfoBar.prototype = {
     },
 
     _hide: function() {
-        this._overviewWasHidden = false;
         Tweener.addTween(this.actor,
                          { opacity: 0,
                            transition: 'easeOutQuad',
@@ -140,31 +135,15 @@ InfoBar.prototype = {
 
     _onTimeout: function() {
         this._timeoutId = 0;
-        if (this._overviewWasHidden)
-            this._hide();
+        this._hide();
         return false;
     },
 
-    _onOverviewHiding: function() {
-        if (this._timeoutId == 0)
-            this._hide();
-        else
-            this._overviewWasHidden = true;
-    },
-
     setMessage: function(text, undoCallback, undoLabel) {
         if (this._timeoutId)
             Mainloop.source_remove(this._timeoutId);
 
-        if (this._hidingOverviewId == 0) {
-            // Set here, because when constructor is called, overview is null.
-            if (!Main.overview)
-                return;
-            // We don't actually use the ID, it's just a way of tracking whether we've hooked up the signal
-            this._hidingOverviewId = Main.overview.connect('hiding', Lang.bind(this, this._onOverviewHiding));
-        }
         this._timeout = false;
-        this._overviewWasHidden = false;
 
         this._label.text = text;
 
@@ -251,19 +230,19 @@ Overview.prototype = {
 
         this._coverPane.lower_bottom();
 
-        this._workspaces = null;
+        this.workspaces = null;
     },
 
     _onViewChanged: function() {
         if (!this.visible)
             return;
 
-        this._workspaces = this._workspacesManager.workspacesView;
+        this.workspaces = this._workspacesManager.workspacesView;
 
         // Show new workspacesView
-        this._group.add_actor(this._workspaces.actor);
-        this._workspacesBar.raise(this._workspaces.actor);
-        this._dash.actor.raise(this._workspaces.actor);
+        this._group.add_actor(this.workspaces.actor);
+        this._workspacesBar.raise(this.workspaces.actor);
+        this._dash.actor.raise(this.workspaces.actor);
     },
 
     _recalculateGridSizes: function () {
@@ -382,7 +361,7 @@ Overview.prototype = {
                         this._activeDisplayPane.close();
                     return true;
                 }));
-                this._workspaces.actor.opacity = 64;
+                this.workspaces.actor.opacity = 64;
             } else if (pane == this._activeDisplayPane) {
                 this._activeDisplayPane = null;
                 if (backgroundEventId != null) {
@@ -391,7 +370,7 @@ Overview.prototype = {
                 }
                 this._transparentBackground.lower_bottom();
                 this._paneContainer.hide();
-                this._workspaces.actor.opacity = 255;
+                this.workspaces.actor.opacity = 255;
             }
         }));
     },
@@ -417,13 +396,13 @@ Overview.prototype = {
     // Returns the scale the Overview has when we just start zooming out
     // to overview mode. That is, when just the active workspace is showing.
     getZoomedInScale : function() {
-        return 1 / this._workspaces.getScale();
+        return 1 / this.workspaces.getScale();
     },
 
     // Returns the position the Overview has when we just start zooming out
     // to overview mode. That is, when just the active workspace is showing.
     getZoomedInPosition : function() {
-        let [posX, posY] = this._workspaces.getActiveWorkspacePosition();
+        let [posX, posY] = this.workspaces.getActiveWorkspacePosition();
         let scale = this.getZoomedInScale();
 
         return [- posX * scale, - posY * scale];
@@ -458,13 +437,13 @@ Overview.prototype = {
                                                  this._workspacesY);
         this._workspacesManager.connect('view-changed',
                                         Lang.bind(this, this._onViewChanged));
-        this._workspaces = this._workspacesManager.workspacesView;
-        this._group.add_actor(this._workspaces.actor);
+        this.workspaces = this._workspacesManager.workspacesView;
+        this._group.add_actor(this.workspaces.actor);
 
         // The workspaces actor is as big as the screen, so we have to raise the dash above it
         // for drag and drop to work.  In the future we should fix the workspaces to not
         // be as big as the screen.
-        this._dash.actor.raise(this._workspaces.actor);
+        this._dash.actor.raise(this.workspaces.actor);
 
         this._workspacesBar = this._workspacesManager.controlsBar.actor;
         this._workspacesBar.set_position(this._workspacesBarX,
@@ -472,7 +451,7 @@ Overview.prototype = {
         this._workspacesBar.width = this._workspacesBarWidth;
 
         this._group.add_actor(this._workspacesBar);
-        this._workspacesBar.raise(this._workspaces.actor);
+        this._workspacesBar.raise(this.workspaces.actor);
 
         // All the the actors in the window group are completely obscured,
         // hiding the group holding them while the Overview is displayed greatly
@@ -522,7 +501,7 @@ Overview.prototype = {
         this._hideInProgress = true;
         if (this._activeDisplayPane != null)
             this._activeDisplayPane.close();
-        this._workspaces.hide();
+        this.workspaces.hide();
 
         // Create a zoom in effect by transforming the Overview group so that
         // the active workspace fills up the whole screen. The opposite
@@ -567,7 +546,7 @@ Overview.prototype = {
      * and will return %null.
      */
     getWorkspacesForWindow: function(metaWindow) {
-        return this._workspaces;
+        return this.workspaces;
     },
 
     //// Private methods ////
@@ -585,8 +564,8 @@ Overview.prototype = {
     _hideDone: function() {
         global.window_group.show();
 
-        this._workspaces.destroy();
-        this._workspaces = null;
+        this.workspaces.destroy();
+        this.workspaces = null;
 
         this._workspacesBar.destroy();
         this._workspacesBar = null;
