@@ -1,6 +1,5 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Big = imports.gi.Big;
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
@@ -52,7 +51,7 @@ CommandCompleter.prototype = {
             this._paths[i] = file.get_path();
             this._monitors[i] = file.monitor_directory(Gio.FileMonitorFlags.NONE, null);
             if (this._monitors[i] != null) {
-                this._monitors[i].connect("changed", Lang.bind(this, this._onChanged));
+                this._monitors[i].connect('changed', Lang.bind(this, this._onChanged));
             }
         }
         this._paths = this._paths.filter(function(a) {
@@ -133,7 +132,7 @@ CommandCompleter.prototype = {
     },
 
     getCompletion: function(text) {
-        let common = "";
+        let common = '';
         let notInit = true;
         if (!this._valid) {
             this._update(0);
@@ -146,7 +145,7 @@ CommandCompleter.prototype = {
                     break;
             }
             if (k == 0)
-                return "";
+                return '';
             return s1.substr(0, k);
         }
         function _hasPrefix(s1, prefix) {
@@ -214,7 +213,7 @@ RunDialog.prototype = {
         // hidden then show it in show()
         this._group = new Clutter.Group({ visible: false,
                                           x: 0, y: 0 });
-        global.stage.add_actor(this._group);
+        Main.uiGroup.add_actor(this._group);
 
         let lightbox = new Lightbox.Lightbox(this._group, true);
 
@@ -268,8 +267,8 @@ RunDialog.prototype = {
                 this._setCommandFromHistory(this._historyIndex--);
                 return true;
             }
-            if (symbol == Clutter.Return) {
-                if (e.get_state() & Clutter.ModifierType.CONTROL_MASK)
+            if (symbol == Clutter.Return || symbol == Clutter.KP_Enter) {
+                if (Shell.get_event_state(e) & Clutter.ModifierType.CONTROL_MASK)
                     this._run(o.get_text(), true);
                 else
                     this._run(o.get_text(), false);
@@ -330,8 +329,10 @@ RunDialog.prototype = {
     _run : function(input, inTerminal) {
         let command = input;
 
-        this._history.push(input);
-        this._saveHistory();
+        if (this._history.length > 0 && this._history[this._history.length - 1] != input) {
+            this._history.push(input);
+            this._saveHistory();
+        }
 
         this._commandError = false;
         let f;
@@ -371,7 +372,7 @@ RunDialog.prototype = {
                     // We are only interested in the actual error, so parse
                     //that out.
                     let m = /.+\((.+)\)/.exec(e);
-                    let errorStr = _("Execution of '%s' failed:").format(command) + "\n" + m[1];
+                    let errorStr = _("Execution of '%s' failed:").format(command) + '\n' + m[1];
                     this._errorMessage.set_text(errorStr);
 
                     this._errorBox.show();
