@@ -5,6 +5,7 @@ const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
 const Shell = imports.gi.Shell;
+const St = imports.gi.St;
 
 const AltTab = imports.ui.altTab;
 const WorkspaceSwitcherPopup = imports.ui.workspaceSwitcherPopup;
@@ -19,8 +20,9 @@ function WindowManager() {
 
 WindowManager.prototype = {
     _init : function() {
-        let shellwm = global.window_manager;
+        this._shellwm =  global.window_manager;
 
+        this._keyBindingHandlers = [];
         this._minimizing = [];
         this._maximizing = [];
         this._unmaximizing = [];
@@ -28,32 +30,35 @@ WindowManager.prototype = {
         this._destroying = [];
 
         this._switchData = null;
-        shellwm.connect('switch-workspace', Lang.bind(this, this._switchWorkspace));
-        shellwm.connect('kill-switch-workspace', Lang.bind(this, this._switchWorkspaceDone));
-        shellwm.connect('minimize', Lang.bind(this, this._minimizeWindow));
-        shellwm.connect('kill-minimize', Lang.bind(this, this._minimizeWindowDone));
-        shellwm.connect('maximize', Lang.bind(this, this._maximizeWindow));
-        shellwm.connect('kill-maximize', Lang.bind(this, this._maximizeWindowDone));
-        shellwm.connect('unmaximize', Lang.bind(this, this._unmaximizeWindow));
-        shellwm.connect('kill-unmaximize', Lang.bind(this, this._unmaximizeWindowDone));
-        shellwm.connect('map', Lang.bind(this, this._mapWindow));
-        shellwm.connect('kill-map', Lang.bind(this, this._mapWindowDone));
-        shellwm.connect('destroy', Lang.bind(this, this._destroyWindow));
-        shellwm.connect('kill-destroy', Lang.bind(this, this._destroyWindowDone));
-
-        shellwm.takeover_keybinding('switch_windows');
-        shellwm.connect('keybinding::switch_windows', Lang.bind(this, this._startAppSwitcher));
+        this._shellwm.connect('switch-workspace', Lang.bind(this, this._switchWorkspace));
+        this._shellwm.connect('kill-switch-workspace', Lang.bind(this, this._switchWorkspaceDone));
+        this._shellwm.connect('minimize', Lang.bind(this, this._minimizeWindow));
+        this._shellwm.connect('kill-minimize', Lang.bind(this, this._minimizeWindowDone));
+        this._shellwm.connect('maximize', Lang.bind(this, this._maximizeWindow));
+        this._shellwm.connect('kill-maximize', Lang.bind(this, this._maximizeWindowDone));
+        this._shellwm.connect('unmaximize', Lang.bind(this, this._unmaximizeWindow));
+        this._shellwm.connect('kill-unmaximize', Lang.bind(this, this._unmaximizeWindowDone));
+        this._shellwm.connect('map', Lang.bind(this, this._mapWindow));
+        this._shellwm.connect('kill-map', Lang.bind(this, this._mapWindowDone));
+        this._shellwm.connect('destroy', Lang.bind(this, this._destroyWindow));
+        this._shellwm.connect('kill-destroy', Lang.bind(this, this._destroyWindowDone));
 
         this._workspaceSwitcherPopup = null;
-        shellwm.takeover_keybinding('switch_to_workspace_left');
-        shellwm.takeover_keybinding('switch_to_workspace_right');
-        shellwm.takeover_keybinding('switch_to_workspace_up');
-        shellwm.takeover_keybinding('switch_to_workspace_down');
-        shellwm.connect('keybinding::switch_to_workspace_left', Lang.bind(this, this._showWorkspaceSwitcher));
-        shellwm.connect('keybinding::switch_to_workspace_right', Lang.bind(this, this._showWorkspaceSwitcher));
-        shellwm.connect('keybinding::switch_to_workspace_up', Lang.bind(this, this._showWorkspaceSwitcher));
-        shellwm.connect('keybinding::switch_to_workspace_down', Lang.bind(this, this._showWorkspaceSwitcher));
+        this.setKeybindingHandler('switch_to_workspace_left', Lang.bind(this, this._showWorkspaceSwitcher));
+        this.setKeybindingHandler('switch_to_workspace_right', Lang.bind(this, this._showWorkspaceSwitcher));
+        this.setKeybindingHandler('switch_to_workspace_up', Lang.bind(this, this._showWorkspaceSwitcher));
+        this.setKeybindingHandler('switch_to_workspace_down', Lang.bind(this, this._showWorkspaceSwitcher));
+        this.setKeybindingHandler('switch_windows', Lang.bind(this, this._startAppSwitcher));
+    },
 
+    setKeybindingHandler: function(keybinding, handler){
+        if (this._keyBindingHandlers[keybinding])
+            this._shellwm.disconnect(this._keyBindingHandlers[keybinding]);
+        else
+            this._shellwm.takeover_keybinding(keybinding);
+
+        this._keyBindingHandlers[keybinding] =
+            this._shellwm.connect('keybinding::' + keybinding, handler);
     },
 
     _shouldAnimate : function(actor) {
@@ -88,14 +93,17 @@ WindowManager.prototype = {
         this._minimizing.push(actor);
 
         let primary = global.get_primary_monitor();
+        let xDest = primary.x;
+        if (St.Widget.get_default_direction() == St.TextDirection.RTL)
+            xDest += primary.width;
 
         Tweener.addTween(actor,
                          { scale_x: 0.0,
                            scale_y: 0.0,
-                           x: primary.x,
+                           x: xDest,
                            y: 0,
                            time: WINDOW_ANIMATION_TIME,
-                           transition: "easeOutQuad",
+                           transition: 'easeOutQuad',
                            onComplete: this._minimizeWindowDone,
                            onCompleteScope: this,
                            onCompleteParams: [shellwm, actor],
@@ -144,17 +152,15 @@ WindowManager.prototype = {
             return;
         }
 
-        actor.move_anchor_point_from_gravity(Clutter.Gravity.CENTER);
-        actor.set_scale(0.0, 0.0);
+        actor.opacity = 0;
         actor.show();
-        
-        /* scale window up from 0x0 to normal size */
+
+        /* Fade window in */
         this._mapping.push(actor);
         Tweener.addTween(actor,
-                         { scale_x: 1.0,
-                           scale_y: 1.0,
+                         { opacity: 255,
                            time: WINDOW_ANIMATION_TIME,
-                           transition: "easeOutQuad",
+                           transition: 'easeOutQuad',
                            onComplete: this._mapWindowDone,
                            onCompleteScope: this,
                            onCompleteParams: [shellwm, actor],
@@ -167,8 +173,7 @@ WindowManager.prototype = {
     _mapWindowDone : function(shellwm, actor) {
         if (this._removeEffect(this._mapping, actor)) {
             Tweener.removeTweens(actor);
-            actor.set_scale(1.0, 1.0);
-            actor.move_anchor_point_from_gravity(Clutter.Gravity.NORTH_WEST);
+            actor.opacity = 255;
             shellwm.completed_map(actor);
         }
     },
@@ -253,7 +258,7 @@ WindowManager.prototype = {
                          { x: xDest,
                            y: yDest,
                            time: WINDOW_ANIMATION_TIME,
-                           transition: "easeOutQuad",
+                           transition: 'easeOutQuad',
                            onComplete: this._switchWorkspaceDone,
                            onCompleteScope: this,
                            onCompleteParams: [shellwm]
@@ -262,7 +267,7 @@ WindowManager.prototype = {
                          { x: 0,
                            y: 0,
                            time: WINDOW_ANIMATION_TIME,
-                           transition: "easeOutQuad"
+                           transition: 'easeOutQuad'
                          });
     },
 
@@ -301,7 +306,7 @@ WindowManager.prototype = {
 
     _showWorkspaceSwitcher : function(shellwm, binding, window, backwards) {
         /* We don't support this kind of layout */
-        if (binding == "switch_to_workspace_up" || binding == "switch_to_workspace_down")
+        if (binding == 'switch_to_workspace_up' || binding == 'switch_to_workspace_down')
             return;
 
         if (global.screen.n_workspaces == 1)
@@ -310,24 +315,34 @@ WindowManager.prototype = {
         if (this._workspaceSwitcherPopup == null)
             this._workspaceSwitcherPopup = new WorkspaceSwitcherPopup.WorkspaceSwitcherPopup();
 
-        let activeWorkspaceIndex = global.screen.get_active_workspace_index();
-
-        if (binding == "switch_to_workspace_left") {
-            if (activeWorkspaceIndex > 0) {
-                global.screen.get_workspace_by_index(activeWorkspaceIndex - 1).activate(global.get_current_time());
-                this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.LEFT, activeWorkspaceIndex - 1);
-            }
-            else
-                this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.LEFT, activeWorkspaceIndex);
+        if (binding == 'switch_to_workspace_left') {
+            this.actionMoveWorkspaceLeft();
         }
 
-        if (binding == "switch_to_workspace_right") {
-            if (activeWorkspaceIndex <  global.screen.n_workspaces - 1) {
-                global.screen.get_workspace_by_index(activeWorkspaceIndex + 1).activate(global.get_current_time());
+        if (binding == 'switch_to_workspace_right') {
+            this.actionMoveWorkspaceRight();
+        }
+    },
+
+    actionMoveWorkspaceLeft: function() {
+        let activeWorkspaceIndex = global.screen.get_active_workspace_index();
+        if (activeWorkspaceIndex > 0) {
+            global.screen.get_workspace_by_index(activeWorkspaceIndex - 1).activate(global.get_current_time());
+            if (!Main.overview.visible)
+                this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.LEFT, activeWorkspaceIndex - 1);
+        } else if (!Main.overview.visible) {
+            this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.LEFT, activeWorkspaceIndex);
+        }
+    },
+
+    actionMoveWorkspaceRight: function() {
+        let activeWorkspaceIndex = global.screen.get_active_workspace_index();
+        if (activeWorkspaceIndex <  global.screen.n_workspaces - 1) {
+            global.screen.get_workspace_by_index(activeWorkspaceIndex + 1).activate(global.get_current_time());
+            if (!Main.overview.visible)
                 this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.RIGHT, activeWorkspaceIndex + 1);
-            }
-            else
-                this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.RIGHT, activeWorkspaceIndex);
+        } else if (!Main.overview.visible) {
+            this._workspaceSwitcherPopup.display(WorkspaceSwitcherPopup.RIGHT, activeWorkspaceIndex);
         }
     }
 };
