@@ -23,6 +23,8 @@
  * 02111-1307, USA.
  */
 
+#include "config.h"
+
 #define MUTTER_BUILDING_PLUGIN 1
 #include <mutter-plugin.h>
 
@@ -30,23 +32,27 @@
 
 #include <clutter/clutter.h>
 #include <clutter/x11/clutter-x11.h>
+#include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gjs/gjs.h>
 #include <girepository.h>
 #include <gmodule.h>
+#include <malloc.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "display.h"
 
-#include "shell-global.h"
+#include "shell-global-private.h"
+#include "shell-perf-log.h"
 #include "shell-wm.h"
+#include "st.h"
 
-static void gnome_shell_plugin_constructed (GObject *object);
 static void gnome_shell_plugin_dispose     (GObject *object);
 static void gnome_shell_plugin_finalize    (GObject *object);
 
+static void     gnome_shell_plugin_start            (MutterPlugin         *plugin);
 static void     gnome_shell_plugin_minimize         (MutterPlugin         *plugin,
                                                      MutterWindow         *actor);
 static void     gnome_shell_plugin_maximize         (MutterPlugin         *plugin,
@@ -116,10 +122,10 @@ gnome_shell_plugin_class_init (GnomeShellPluginClass *klass)
   GObjectClass      *gobject_class = G_OBJECT_CLASS (klass);
   MutterPluginClass *plugin_class  = MUTTER_PLUGIN_CLASS (klass);
 
-  gobject_class->constructed     = gnome_shell_plugin_constructed;
   gobject_class->dispose         = gnome_shell_plugin_dispose;
   gobject_class->finalize        = gnome_shell_plugin_finalize;
 
+  plugin_class->start            = gnome_shell_plugin_start;
   plugin_class->map              = gnome_shell_plugin_map;
   plugin_class->minimize         = gnome_shell_plugin_minimize;
   plugin_class->maximize         = gnome_shell_plugin_maximize;
@@ -136,25 +142,23 @@ gnome_shell_plugin_class_init (GnomeShellPluginClass *klass)
 static void
 gnome_shell_plugin_init (GnomeShellPlugin *shell_plugin)
 {
-  _shell_global_set_plugin (shell_global_get(), MUTTER_PLUGIN(shell_plugin));
+  meta_prefs_override_preference_location ("/apps/metacity/general/button_layout",
+                                           "/desktop/gnome/shell/windows/button_layout");
 }
 
 static void
-gnome_shell_plugin_constructed (GObject *object)
+update_font_options (GtkSettings *settings)
 {
-  MutterPlugin *plugin = MUTTER_PLUGIN (object);
-  GnomeShellPlugin *shell_plugin = GNOME_SHELL_PLUGIN (object);
-  MetaScreen *screen;
-  MetaDisplay *display;
-  GError *error = NULL;
-  int status;
-  const char *shell_js;
-  char **search_path;
+  StThemeContext *context;
+  ClutterStage *stage;
   ClutterBackend *backend;
-  cairo_font_options_t *font_options;
-
-  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  gint dpi;
+  gint hinting;
+  gchar *hint_style_str;
+  cairo_hint_style_t hint_style = CAIRO_HINT_STYLE_NONE;
+  gint antialias;
+  cairo_antialias_t antialias_mode = CAIRO_ANTIALIAS_NONE;
+  cairo_font_options_t *options;
 
   /* Disable text mipmapping; it causes problems on pre-GEM Intel
    * drivers and we should just be rendering text at the right
@@ -163,21 +167,145 @@ gnome_shell_plugin_constructed (GObject *object)
    */
   clutter_set_font_flags (clutter_get_font_flags () & ~CLUTTER_FONT_MIPMAPPING);
 
+  g_object_get (settings,
+                "gtk-xft-dpi", &dpi,
+                "gtk-xft-antialias", &antialias,
+                "gtk-xft-hinting", &hinting,
+                "gtk-xft-hintstyle", &hint_style_str,
+                NULL);
+
+  stage = CLUTTER_STAGE (clutter_stage_get_default ());
+  context = st_theme_context_get_for_stage (stage);
+
+  if (dpi != -1)
+    /* GTK stores resolution as 1024 * dots/inch */
+    st_theme_context_set_resolution (context, dpi / 1024);
+  else
+    st_theme_context_set_default_resolution (context);
+
   /* Clutter (as of 0.9) passes comprehensively wrong font options
    * override whatever set_font_flags() did above.
    *
    * http://bugzilla.openedhand.com/show_bug.cgi?id=1456
    */
   backend = clutter_get_default_backend ();
-  font_options = cairo_font_options_create ();
-  /* Default options for everything is reasonable; except that
-   * we want to turn off subpixel anti-aliasing; since Clutter
+  options = cairo_font_options_create ();
+
+  cairo_font_options_set_hint_metrics (options, CAIRO_HINT_METRICS_ON);
+
+  if (hinting >= 0 && !hinting)
+    {
+      hint_style = CAIRO_HINT_STYLE_NONE;
+    }
+  else if (hint_style_str)
+    {
+      if (strcmp (hint_style_str, "hintnone") == 0)
+        hint_style = CAIRO_HINT_STYLE_NONE;
+      else if (strcmp (hint_style_str, "hintslight") == 0)
+        hint_style = CAIRO_HINT_STYLE_SLIGHT;
+      else if (strcmp (hint_style_str, "hintmedium") == 0)
+        hint_style = CAIRO_HINT_STYLE_MEDIUM;
+      else if (strcmp (hint_style_str, "hintfull") == 0)
+        hint_style = CAIRO_HINT_STYLE_FULL;
+    }
+
+  g_free (hint_style_str);
+
+  cairo_font_options_set_hint_style (options, hint_style);
+
+  /* We don't want to turn on subpixel anti-aliasing; since Clutter
    * doesn't currently have the code to support ARGB masks,
    * generating them then squashing them back to A8 is pointless.
    */
-  cairo_font_options_set_antialias (font_options, CAIRO_ANTIALIAS_GRAY);
-  clutter_backend_set_font_options (backend, font_options);
-  cairo_font_options_destroy (font_options);
+  antialias_mode = (antialias < 0 || antialias) ? CAIRO_ANTIALIAS_GRAY
+                                                : CAIRO_ANTIALIAS_NONE;
+
+  cairo_font_options_set_antialias (options, antialias_mode);
+
+  clutter_backend_set_font_options (backend, options);
+  cairo_font_options_destroy (options);
+}
+
+static void
+settings_notify_cb (GtkSettings *settings,
+                    GParamSpec  *pspec,
+                    gpointer     data)
+{
+  update_font_options (settings);
+}
+
+static void
+malloc_statistics_callback (ShellPerfLog *perf_log,
+                            gpointer      data)
+{
+  struct mallinfo info = mallinfo ();
+
+  shell_perf_log_update_statistic_i (perf_log,
+                                     "malloc.arenaSize",
+                                     info.arena);
+  shell_perf_log_update_statistic_i (perf_log,
+                                     "malloc.mmapSize",
+                                     info.hblkhd);
+  shell_perf_log_update_statistic_i (perf_log,
+                                     "malloc.usedSize",
+                                     info.uordblks);
+}
+
+static void
+add_statistics (GnomeShellPlugin *shell_plugin)
+{
+  ShellPerfLog *perf_log = shell_perf_log_get_default ();
+
+  /* For probably historical reasons, mallinfo() defines the returned values,
+   * even those in bytes as int, not size_t. We're determined not to use
+   * more than 2G of malloc'ed memory, so are OK with that.
+   */
+  shell_perf_log_define_statistic (perf_log,
+                                   "malloc.arenaSize",
+                                   "Amount of memory allocated by malloc() with brk(), in bytes",
+                                   "i");
+  shell_perf_log_define_statistic (perf_log,
+                                   "malloc.mmapSize",
+                                   "Amount of memory allocated by malloc() with mmap(), in bytes",
+                                   "i");
+  shell_perf_log_define_statistic (perf_log,
+                                   "malloc.usedSize",
+                                   "Amount of malloc'ed memory currently in use",
+                                   "i");
+
+  shell_perf_log_add_statistics_callback (perf_log,
+                                          malloc_statistics_callback,
+                                          NULL, NULL);
+}
+
+static void
+gnome_shell_plugin_start (MutterPlugin *plugin)
+{
+  GnomeShellPlugin *shell_plugin = GNOME_SHELL_PLUGIN (plugin);
+  MetaScreen *screen;
+  MetaDisplay *display;
+  GtkSettings *settings;
+  GError *error = NULL;
+  int status;
+  const char *shell_js;
+  char **search_path;
+  ShellGlobal *global;
+
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
+  settings = gtk_settings_get_default ();
+  g_object_connect (settings,
+                    "signal::notify::gtk-xft-dpi",
+                    G_CALLBACK (settings_notify_cb), NULL,
+                    "signal::notify::gtk-xft-antialias",
+                    G_CALLBACK (settings_notify_cb), NULL,
+                    "signal::notify::gtk-xft-hinting",
+                    G_CALLBACK (settings_notify_cb), NULL,
+                    "signal::notify::gtk-xft-hintstyle",
+                    G_CALLBACK (settings_notify_cb), NULL,
+                    NULL);
+  update_font_options (settings);
 
   screen = mutter_plugin_get_screen (plugin);
   display = meta_screen_get_display (screen);
@@ -192,6 +320,14 @@ gnome_shell_plugin_constructed (GObject *object)
   shell_plugin->gjs_context = gjs_context_new_with_search_path(search_path);
   g_strfreev(search_path);
 
+  /* Initialize the global object here. */
+  global = shell_global_get ();
+
+  _shell_global_set_plugin (global, MUTTER_PLUGIN(shell_plugin));
+  _shell_global_set_gjs_context (global, shell_plugin->gjs_context);
+
+  add_statistics (shell_plugin);
+
   if (!gjs_context_eval (shell_plugin->gjs_context,
                          "const Main = imports.ui.main; Main.start();",
                          -1,
