@@ -14,7 +14,7 @@
 
 #include "shell-app-private.h"
 #include "shell-global.h"
-#include "shell-texture-cache.h"
+#include "st/st-texture-cache.h"
 #include "display.h"
 
 #define GMENU_I_KNOW_THIS_IS_UNSTABLE
@@ -27,7 +27,8 @@
  */
 static const char*const known_vendor_prefixes[] = { "gnome",
                                                     "fedora",
-                                                    "mozilla" };
+                                                    "mozilla",
+                                                    NULL };
 
 enum {
    PROP_0,
@@ -308,9 +309,15 @@ reread_entries (ShellAppSystem     *self,
   g_slist_free (*cache);
   *cache = NULL;
 
-  *cache = gather_entries_recurse (self, *cache, unique, trunk);
-
-  gmenu_tree_item_unref (trunk);
+  if (!trunk)
+    {
+      *cache = NULL;
+    }
+  else
+    {
+      *cache = gather_entries_recurse (self, *cache, unique, trunk);
+      gmenu_tree_item_unref (trunk);
+    }
 }
 
 static void
@@ -484,6 +491,49 @@ shell_app_system_get_app (ShellAppSystem   *self,
   return app;
 }
 
+/**
+ * shell_app_system_get_app_for_path:
+ * @system: a #ShellAppSystem
+ * @desktop_path: (utf8): UTF-8 encoded absolute file name
+ *
+ * Find or create a #ShellApp corresponding to a given absolute
+ * file name which must be in the standard paths (XDG_DATA_DIRS).
+ * For files outside the datadirs, this function returns %NULL.
+ *
+ * If already cached elsewhere in memory, return that instance.
+ * Otherwise, create a new one.
+ *
+ * Return value: (transfer full): The #ShellApp for id, or %NULL if none
+ */
+ShellApp *
+shell_app_system_get_app_for_path (ShellAppSystem   *system,
+                                   const char       *desktop_path)
+{
+  const char *basename;
+  ShellAppInfo *info;
+
+  basename = g_strrstr (desktop_path, "/");
+  if (basename)
+    basename += 1;
+  else
+    basename = desktop_path;
+
+  info = g_hash_table_lookup (system->priv->app_id_to_info, basename);
+  if (!info)
+    return NULL;
+
+  if (info->type == SHELL_APP_INFO_TYPE_ENTRY)
+    {
+      const char *full_path = gmenu_tree_entry_get_desktop_file_path ((GMenuTreeEntry*) info->entry);
+      if (strcmp (desktop_path, full_path) != 0)
+        return NULL;
+    }
+  else
+    return NULL;
+
+  return shell_app_system_get_app (system, basename);
+}
+
 /**
  * shell_app_system_get_app_for_window:
  * @self: A #ShellAppSystem
@@ -1021,7 +1071,16 @@ themed_icon_from_name (const char *iconname)
   return icon;
 }
 
-static GIcon *
+/**
+ * shell_app_info_get_icon:
+ * @info: A #ShellAppInfo
+ *
+ * Get the #GIcon associated with this app; for apps "faked" from a #MetaWindow,
+ * return %NULL.
+ *
+ * Returns: (transfer full): The icon for @info, or %NULL
+ */
+GIcon *
 shell_app_info_get_icon (ShellAppInfo *info)
 {
   char *iconname = NULL;
@@ -1100,9 +1159,9 @@ shell_app_info_create_icon_texture (ShellAppInfo *info, float size)
 
   if (info->type == SHELL_APP_INFO_TYPE_WINDOW)
     {
-      return shell_texture_cache_bind_pixbuf_property (shell_texture_cache_get_default (),
-                                                       G_OBJECT (info->window),
-                                                       "icon");
+      return st_texture_cache_bind_pixbuf_property (st_texture_cache_get_default (),
+                                                    G_OBJECT (info->window),
+                                                    "icon");
     }
 
   icon = shell_app_info_get_icon (info);
@@ -1113,13 +1172,28 @@ shell_app_info_create_icon_texture (ShellAppInfo *info, float size)
     }
   else
     {
-      ret = shell_texture_cache_load_gicon (shell_texture_cache_get_default (), icon, (int)size);
+      ret = st_texture_cache_load_gicon (st_texture_cache_get_default (), icon, (int)size);
       g_object_unref (icon);
     }
 
   return ret;
 }
 
+/**
+ * shell_app_info_get_source_window:
+ * @info: A #ShellAppInfo
+ *
+ * If @info is tracking a #MetaWindow, return that window.
+ * Otherwise, return %NULL.
+ */
+MetaWindow *
+shell_app_info_get_source_window (ShellAppInfo *info)
+{
+  if (info->type == SHELL_APP_INFO_TYPE_WINDOW)
+    return info->window;
+  return NULL;
+}
+
 /**
  * shell_app_info_launch_full:
  * @timestamp: Event timestamp, or 0 for current event timestamp
@@ -1157,10 +1231,16 @@ shell_app_info_launch_full (ShellAppInfo *info,
       meta_window_activate (info->window, timestamp);
       return TRUE;
     }
-
-  filename = shell_app_info_get_desktop_file_path (info);
-  gapp = g_desktop_app_info_new_from_filename (filename);
-  g_free (filename);
+  else if (info->type == SHELL_APP_INFO_TYPE_ENTRY)
+    {
+      gapp = g_desktop_app_info_new (shell_app_info_get_id (info));
+    }
+  else
+    {
+      filename = shell_app_info_get_desktop_file_path (info);
+      gapp = g_desktop_app_info_new_from_filename (filename);
+      g_free (filename);
+    }
 
   if (!gapp)
     {
@@ -1175,11 +1255,6 @@ shell_app_info_launch_full (ShellAppInfo *info,
   if (timestamp == 0)
     timestamp = clutter_get_current_event_time ();
 
-  /* Shell design calls for on application launch, no window is focused,
-   * and we have startup notification displayed.
-   */
-  meta_display_focus_the_no_focus_window (display, screen, timestamp);
-
   if (workspace < 0)
     workspace = meta_screen_get_active_workspace_index (screen);
 
