@@ -3,6 +3,7 @@
 #include "config.h"
 
 #include "shell-global-private.h"
+#include "shell-perf-log.h"
 #include "shell-wm.h"
 
 #include "display.h"
@@ -26,6 +27,7 @@
 #endif
 
 #define SHELL_DBUS_SERVICE "org.gnome.Shell"
+#define MAGNIFIER_DBUS_SERVICE "org.gnome.Magnifier"
 
 static void grab_notify (GtkWidget *widget, gboolean is_grab, gpointer user_data);
 static void update_root_window_pixmap (ShellGlobal *global);
@@ -50,12 +52,16 @@ struct _ShellGlobal {
   ShellWM *wm;
   const char *datadir;
   const char *imagedir;
-  const char *configdir;
+  const char *userdatadir;
 
   /* Displays the root window; see shell_global_create_root_pixmap_actor() */
   ClutterActor *root_pixmap;
 
   gint last_change_screen_width, last_change_screen_height;
+
+  guint work_count;
+  GSList *leisure_closures;
+  guint leisure_function_id;
 };
 
 enum {
@@ -70,7 +76,7 @@ enum {
   PROP_WINDOW_MANAGER,
   PROP_DATADIR,
   PROP_IMAGEDIR,
-  PROP_CONFIGDIR,
+  PROP_USERDATADIR,
 };
 
 /* Signals */
@@ -145,8 +151,8 @@ shell_global_get_property(GObject         *object,
     case PROP_IMAGEDIR:
       g_value_set_string (value, global->imagedir);
       break;
-    case PROP_CONFIGDIR:
-      g_value_set_string (value, global->configdir);
+    case PROP_USERDATADIR:
+      g_value_set_string (value, global->userdatadir);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -159,7 +165,6 @@ shell_global_init (ShellGlobal *global)
 {
   const char *datadir = g_getenv ("GNOME_SHELL_DATADIR");
   char *imagedir;
-  GFile *conf_dir;
 
   if (!datadir)
     datadir = GNOME_SHELL_DATADIR;
@@ -179,10 +184,8 @@ shell_global_init (ShellGlobal *global)
     }
 
   /* Ensure config dir exists for later use */
-  global->configdir = g_build_filename (g_get_home_dir (), ".gnome2", "shell", NULL);
-  conf_dir = g_file_new_for_path (global->configdir);
-  g_file_make_directory (conf_dir, NULL, NULL);
-  g_object_unref (conf_dir);
+  global->userdatadir = g_build_filename (g_get_user_data_dir (), "gnome-shell", NULL);
+  g_mkdir_with_parents (global->userdatadir, 0700);
   
   global->grab_notifier = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
   g_signal_connect (global->grab_notifier, "grab-notify", G_CALLBACK (grab_notify), global);
@@ -278,10 +281,10 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                         NULL,
                                                         G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
-                                   PROP_CONFIGDIR,
-                                   g_param_spec_string ("configdir",
-                                                        "Configuration directory",
-                                                        "Directory containing gnome-shell configuration files",
+                                   PROP_USERDATADIR,
+                                   g_param_spec_string ("userdatadir",
+                                                        "User data directory",
+                                                        "Directory containing gnome-shell user data",
                                                         NULL,
                                                         G_PARAM_READABLE));
 }
@@ -438,24 +441,33 @@ shell_global_get_windows (ShellGlobal *global)
 }
 
 static gboolean
-on_screen_size_changed_cb (gpointer data)
+update_screen_size (ShellGlobal *global)
 {
-  ShellGlobal *global = SHELL_GLOBAL (data);
-
   int width, height;
 
   mutter_plugin_query_screen_size (global->plugin, &width, &height);
 
-  if (global->last_change_screen_width != width || global->last_change_screen_height != height)
-    {
-      g_signal_emit (G_OBJECT (global), shell_global_signals[SCREEN_SIZE_CHANGED], 0);
-      global->last_change_screen_width = width;
-      global->last_change_screen_height = height;
+  if (global->last_change_screen_width == width && global->last_change_screen_height == height)
+    return FALSE;
 
-      /* update size of background actor to fix tiled backgrounds */
-      clutter_actor_set_size (CLUTTER_ACTOR (global->root_pixmap),
-                              width, height);
-    }
+  global->last_change_screen_width = width;
+  global->last_change_screen_height = height;
+
+  /* update size of background actor to fix tiled backgrounds */
+  if (global->root_pixmap)
+    clutter_actor_set_size (CLUTTER_ACTOR (global->root_pixmap),
+                            width, height);
+
+  return TRUE;
+}
+
+static gboolean
+on_screen_size_changed_cb (gpointer data)
+{
+  ShellGlobal *global = SHELL_GLOBAL (data);
+
+  if (update_screen_size (global))
+    g_signal_emit (G_OBJECT (global), shell_global_signals[SCREEN_SIZE_CHANGED], 0);
 
   return FALSE;
 }
@@ -491,36 +503,54 @@ global_stage_notify_height (GObject    *gobject,
 }
 
 static void
-global_plugin_notify_screen (GObject    *gobject,
-                             GParamSpec *pspec,
-                             gpointer    data)
+global_stage_before_paint (ClutterStage *stage,
+                           ShellGlobal  *global)
 {
-  ShellGlobal *global = SHELL_GLOBAL (data);
-  ClutterActor *stage = mutter_plugin_get_stage (MUTTER_PLUGIN (gobject));
+  shell_perf_log_event (shell_perf_log_get_default (),
+                        "clutter.stagePaintStart");
+}
 
-  g_signal_connect (stage, "notify::width",
-                    G_CALLBACK (global_stage_notify_width), global);
-  g_signal_connect (stage, "notify::height",
-                    G_CALLBACK (global_stage_notify_height), global);
+static void
+global_stage_after_paint (ClutterStage *stage,
+                          ShellGlobal  *global)
+{
+  shell_perf_log_event (shell_perf_log_get_default (),
+                        "clutter.stagePaintDone");
 }
 
 void
 _shell_global_set_plugin (ShellGlobal  *global,
                           MutterPlugin *plugin)
 {
+  ClutterActor *stage;
+
   g_return_if_fail (SHELL_IS_GLOBAL (global));
   g_return_if_fail (global->plugin == NULL);
 
   global->plugin = plugin;
   global->wm = shell_wm_new (plugin);
 
-  /* At this point screen is NULL, so we can't yet do signal connections
-   * to the width and height; we wait until the screen property is set
-   * to do that. Note that this is a one time thing - screen will never
-   * change once first set.
-   */
-  g_signal_connect (plugin, "notify::screen",
-                    G_CALLBACK (global_plugin_notify_screen), global);
+  stage = mutter_plugin_get_stage (plugin);
+
+  g_signal_connect (stage, "notify::width",
+                    G_CALLBACK (global_stage_notify_width), global);
+  g_signal_connect (stage, "notify::height",
+                    G_CALLBACK (global_stage_notify_height), global);
+  update_screen_size (global);
+
+  g_signal_connect (stage, "paint",
+                    G_CALLBACK (global_stage_before_paint), global);
+  g_signal_connect_after (stage, "paint",
+                          G_CALLBACK (global_stage_after_paint), global);
+
+  shell_perf_log_define_event (shell_perf_log_get_default(),
+                               "clutter.stagePaintStart",
+                               "Start of stage page repaint",
+                               "");
+  shell_perf_log_define_event (shell_perf_log_get_default(),
+                               "clutter.stagePaintDone",
+                               "End of stage page repaint",
+                               "");
 }
 
 void
@@ -811,6 +841,22 @@ shell_global_gc (ShellGlobal *global)
   JS_GC (context);
 }
 
+/**
+ * shell_global_maybe_gc:
+ * @global: A #ShellGlobal
+ *
+ * Start a garbage collection process when it would free up enough memory
+ * to be worth the amount of time it would take
+ * https://developer.mozilla.org/en/SpiderMonkey/JSAPI_Reference/JS_MaybeGC
+ */
+void
+shell_global_maybe_gc (ShellGlobal *global)
+{
+  JSContext *context = gjs_context_get_native_context (global->js_context);
+
+  JS_MaybeGC (context);
+}
+
 void
 shell_global_grab_dbus_service (ShellGlobal *global)
 {
@@ -856,10 +902,24 @@ shell_global_grab_dbus_service (ShellGlobal *global)
           exit (1);
         }
     }
+
+  /* ...and the org.gnome.Magnifier service.
+   */
+  if (!dbus_g_proxy_call (bus, "RequestName", &error,
+                          G_TYPE_STRING, MAGNIFIER_DBUS_SERVICE,
+                          G_TYPE_UINT, 0,
+                          G_TYPE_INVALID,
+                          G_TYPE_UINT, &request_name_result,
+                          G_TYPE_INVALID))
+    {
+      g_print ("failed to acquire %s: %s\n", MAGNIFIER_DBUS_SERVICE, error->message);
+      /* Failing to acquire the magnifer service is not fatal.  Log the error,
+       * but keep going. */
+    }
   g_object_unref (bus);
 }
 
-static void 
+static void
 grab_notify (GtkWidget *widget, gboolean was_grabbed, gpointer user_data)
 {
   ShellGlobal *global = SHELL_GLOBAL (user_data);
@@ -1011,6 +1071,10 @@ shell_global_create_root_pixmap_actor (ShellGlobal *global)
     {
       global->root_pixmap = clutter_glx_texture_pixmap_new ();
 
+      clutter_actor_set_size (CLUTTER_ACTOR (global->root_pixmap),
+                              global->last_change_screen_width,
+                              global->last_change_screen_height);
+
       clutter_texture_set_repeat (CLUTTER_TEXTURE (global->root_pixmap),
                                   TRUE, TRUE);
 
@@ -1103,16 +1167,40 @@ shell_global_get_monitors (ShellGlobal *global)
 GdkRectangle *
 shell_global_get_primary_monitor (ShellGlobal  *global)
 {
-  MetaScreen *screen = shell_global_get_screen (global);
-  MetaRectangle rect;
+  GdkScreen *screen = gdk_screen_get_default ();
+  GdkRectangle rect;
+  gint primary = 0;
 
-  g_assert (sizeof (MetaRectangle) == sizeof (GdkRectangle) &&
-            G_STRUCT_OFFSET (MetaRectangle, x) == G_STRUCT_OFFSET (GdkRectangle, x) &&
-            G_STRUCT_OFFSET (MetaRectangle, y) == G_STRUCT_OFFSET (GdkRectangle, y) &&
-            G_STRUCT_OFFSET (MetaRectangle, width) == G_STRUCT_OFFSET (GdkRectangle, width) &&
-            G_STRUCT_OFFSET (MetaRectangle, height) == G_STRUCT_OFFSET (GdkRectangle, height));
+  /* gdk_screen_get_primary_monitor is only present in gtk-2.20+
+   * and is in a useable state (supports heuristics and fallback modes)
+   * starting with 2.20.1
+   */
+#if !GTK_CHECK_VERSION (2, 20, 1)
+  gint i;
+  gchar *output_name = NULL;
+  gint num_monitors = gdk_screen_get_n_monitors (screen);
+
+  for (i = 0; i < num_monitors; i++)
+    {
+      /* Prefer the laptop's internal screen if present */
+      output_name = gdk_screen_get_monitor_plug_name (screen, i);
+      if (output_name)
+        {
+          gboolean is_lvds = g_ascii_strncasecmp (output_name, "LVDS", 4) == 0;
+          g_free (output_name);
+          if (is_lvds)
+            {
+              primary = i;
+              break;
+            }
+        }
+    }
+#else
+  primary = gdk_screen_get_primary_monitor (screen);
+#endif
+
+  gdk_screen_get_monitor_geometry (screen, primary, &rect);
 
-  meta_screen_get_monitor_geometry (screen, 0, &rect);
   return g_boxed_copy (GDK_TYPE_RECTANGLE, &rect);
 }
 
@@ -1144,7 +1232,7 @@ shell_global_get_focus_monitor (ShellGlobal  *global)
         {
           meta_screen_get_monitor_geometry (screen, i, &rect);
 
-          if (rect.x < wrect.x && rect.y < wrect.y &&
+          if (rect.x <= wrect.x && rect.y <= wrect.y &&
               rect.x + rect.width > wrect.x &&
               rect.y + rect.height > wrect.y)
             return g_boxed_copy (GDK_TYPE_RECTANGLE, &rect);
@@ -1156,23 +1244,27 @@ shell_global_get_focus_monitor (ShellGlobal  *global)
 }
 
 /**
- * shell_global_get_modifier_keys:
+ * shell_global_get_pointer:
  * @global: the #ShellGlobal
+ * @x: (out): the X coordinate of the pointer, in global coordinates
+ * @y: (out): the Y coordinate of the pointer, in global coordinates
+ * @mods: (out): the current set of modifier keys that are pressed down
  *
- * Gets the current set of modifier keys that are pressed down;
- * this is a wrapper around gdk_display_get_pointer() that strips
+ * Gets the pointer coordinates and current modifier key state.
+ * This is a wrapper around gdk_display_get_pointer() that strips
  * out any un-declared modifier flags, to make gjs happy; see
  * https://bugzilla.gnome.org/show_bug.cgi?id=597292.
- *
- * Return value: the current modifiers
  */
-GdkModifierType
-shell_global_get_modifier_keys (ShellGlobal *global)
+void
+shell_global_get_pointer (ShellGlobal         *global,
+                          int                 *x,
+                          int                 *y,
+                          ClutterModifierType *mods)
 {
-  GdkModifierType mods;
+  GdkModifierType raw_mods;
 
-  gdk_display_get_pointer (gdk_display_get_default (), NULL, NULL, NULL, &mods);
-  return mods & GDK_MODIFIER_MASK;
+  gdk_display_get_pointer (gdk_display_get_default (), NULL, x, y, &raw_mods);
+  *mods = raw_mods & GDK_MODIFIER_MASK;
 }
 
 /**
@@ -1227,6 +1319,26 @@ shell_popup_menu (GtkMenu *menu, int button, guint32 time,
                   button, time);
 }
 
+/**
+ * shell_write_string_to_stream:
+ * @stream: a #GOutputStream
+ * @str: a UTF-8 string to write to @stream
+ * @error: location to store GError
+ *
+ * Write a string to a GOutputStream as UTF-8. This is a workaround
+ * for not having binary buffers in GJS.
+ *
+ * Return value: %TRUE if write succeeded
+ */
+gboolean
+shell_write_string_to_stream (GOutputStream *stream,
+                              const char    *str,
+                              GError       **error)
+{
+  return g_output_stream_write_all (stream, str, strlen (str),
+                                    NULL, NULL, error);
+}
+
 /**
  * shell_global_get_current_time:
  * @global: A #ShellGlobal
@@ -1286,3 +1398,209 @@ shell_global_create_app_launch_context (ShellGlobal *global)
 
   return (GAppLaunchContext *)context;
 }
+
+/**
+ * shell_global_set_property_mutable:
+ * @global: the #ShellGlobal
+ * @object: the "path" to a JS object, starting from the root object.
+ *  (Eg, "global.stage" or "imports.gi.Gtk.Window.prototype")
+ * @property: a property on @object
+ * @mutable: %TRUE or %FALSE
+ *
+ * If @mutable is %TRUE, this clears the "permanent" and "readonly" flags
+ * on @property of @object. If @mutable is %FALSE, it sets them.
+ *
+ * You can use this to make it possible to modify properties that
+ * would otherwise be read-only from JavaScript.
+ *
+ * Return value: success or failure.
+ */
+gboolean
+shell_global_set_property_mutable (ShellGlobal *global,
+                                   const char  *object,
+                                   const char  *property,
+                                   gboolean     mutable)
+{
+  JSContext *context = gjs_context_get_native_context (global->js_context);
+  char **parts;
+  JSObject *obj;
+  jsval val = JSVAL_VOID;
+  int i;
+  jsuint attrs;
+  JSBool found;
+
+  JS_AddRoot (context, &val);
+
+  parts = g_strsplit (object, ".", -1);
+  obj = JS_GetGlobalObject (context);
+  for (i = 0; parts[i]; i++)
+    {
+      if (!JS_GetProperty (context, obj, parts[i], &val))
+        {
+          g_strfreev (parts);
+          JS_RemoveRoot (context, &val);
+          gjs_log_exception (context, NULL);
+          return FALSE;
+        }
+      obj = JSVAL_TO_OBJECT (val);
+    }
+  g_strfreev (parts);
+
+  if (!JS_GetPropertyAttributes (context, obj, property, &attrs, &found) || !found)
+    {
+      JS_RemoveRoot (context, &val);
+      gjs_log_exception (context, NULL);
+      return FALSE;
+    }
+
+  if (mutable)
+    attrs &= ~(JSPROP_PERMANENT | JSPROP_READONLY);
+  else
+    attrs |= (JSPROP_PERMANENT | JSPROP_READONLY);
+
+  JS_SetPropertyAttributes (context, obj, property, attrs, &found);
+
+  JS_RemoveRoot (context, &val);
+  return !gjs_log_exception (context, NULL);
+}
+
+typedef struct
+{
+  ShellLeisureFunction func;
+  gpointer user_data;
+  GDestroyNotify notify;
+} LeisureClosure;
+
+static gboolean
+run_leisure_functions (gpointer data)
+{
+  ShellGlobal *global = data;
+  GSList *closures;
+  GSList *iter;
+
+  global->leisure_function_id = 0;
+
+  /* We started more work since we scheduled the idle */
+  if (global->work_count > 0)
+    return FALSE;
+
+  /*
+   * We do call MAYBE_GC() here to free up some memory and
+   * prevent the GC from running when we are busy doing other things.
+   */
+  shell_global_maybe_gc (global);
+
+  /* No leisure closures, so we are done */
+  if (global->leisure_closures == NULL)
+    return FALSE;
+
+  closures = global->leisure_closures;
+  global->leisure_closures = NULL;
+
+  for (iter = closures; iter; iter = iter->next)
+    {
+      LeisureClosure *closure = closures->data;
+      closure->func (closure->user_data);
+
+      if (closure->notify)
+        closure->notify (closure->user_data);
+
+      g_slice_free (LeisureClosure, closure);
+    }
+
+  g_slist_free (closures);
+
+  return FALSE;
+}
+
+static void
+schedule_leisure_functions (ShellGlobal *global)
+{
+  /* This is called when we think we are ready to run leisure functions
+   * by our own accounting. We try to handle other types of business
+   * (like ClutterAnimation) by adding a low priority idle function.
+   *
+   * This won't work properly if the mainloop goes idle waiting for
+   * the vertical blanking interval or waiting for work being done
+   * in another thread.
+   */
+  if (!global->leisure_function_id)
+    global->leisure_function_id = g_idle_add_full (G_PRIORITY_LOW,
+                                                   run_leisure_functions,
+                                                   global, NULL);
+}
+
+/**
+ * shell_global_begin_work:
+ * @global: the #ShellGlobal
+ *
+ * Marks that we are currently doing work. This is used to to track
+ * whether we are busy for the purposes of shell_global_run_at_leisure().
+ * A count is kept and shell_global_end_work() must be called exactly
+ * as many times as shell_global_begin_work().
+ */
+void
+shell_global_begin_work (ShellGlobal *global)
+{
+  global->work_count++;
+}
+
+/**
+ * shell_global_end_work:
+ * @global: the #ShellGlobal
+ *
+ * Marks the end of work that we started with shell_global_begin_work().
+ * If no other work is ongoing and functions have been added with
+ * shell_global_run_at_leisure(), they will be run at the next
+ * opportunity.
+ */
+void
+shell_global_end_work (ShellGlobal *global)
+{
+  g_return_if_fail (global->work_count > 0);
+
+  global->work_count--;
+  if (global->work_count == 0)
+    schedule_leisure_functions (global);
+
+}
+
+/**
+ * shell_global_run_at_leisure:
+ * @global: the #ShellGlobal
+ * @func: function to call at leisure
+ * @user_data: data to pass to @func
+ * @notify: function to call to free @user_data
+ *
+ * Schedules a function to be called the next time the shell is idle.
+ * Idle means here no animations, no redrawing, and no ongoing background
+ * work. Since there is currently no way to hook into the Clutter master
+ * clock and know when is running, the implementation here is somewhat
+ * approximation. Animations done through the shell's Tweener module will
+ * be handled properly, but other animations may be detected as terminating
+ * early if they can be drawn fast enough so that the event loop goes idle
+ * between frames.
+ *
+ * The intent of this function is for performance measurement runs
+ * where a number of actions should be run serially and each action is
+ * timed individually. Using this function for other purposes will
+ * interfere with the ability to use it for performance measurement so
+ * should be avoided.
+ */
+void
+shell_global_run_at_leisure (ShellGlobal         *global,
+                             ShellLeisureFunction func,
+                             gpointer             user_data,
+                             GDestroyNotify       notify)
+{
+  LeisureClosure *closure = g_slice_new (LeisureClosure);
+  closure->func = func;
+  closure->user_data = user_data;
+  closure->notify = notify;
+
+  global->leisure_closures = g_slist_append (global->leisure_closures,
+                                             closure);
+
+  if (global->work_count == 0)
+    schedule_leisure_functions (global);
+}
