@@ -16,6 +16,10 @@ G_BEGIN_DECLS
 GType st_table_child_options_get_type (void) G_GNUC_CONST;
 #define ST_TYPE_TABLE_CHILD_OPTIONS (st_table_child_options_get_type())
 
+/* enumerations from "st/st-texture-cache.h" */
+GType st_texture_cache_policy_get_type (void) G_GNUC_CONST;
+#define ST_TYPE_TEXTURE_CACHE_POLICY (st_texture_cache_policy_get_type())
+
 /* enumerations from "st/st-theme-node.h" */
 GType st_side_get_type (void) G_GNUC_CONST;
 #define ST_TYPE_SIDE (st_side_get_type())
