@@ -18,6 +18,8 @@ typedef struct _StShadow StShadow;
  * @yoffset: vertical offset - positive values mean placement below, negative
  *           values placement above the element.
  * @blur: shadow's blur radius - a value of 0.0 will result in a hard shadow.
+ * @spread: shadow's spread radius - grow the shadow without enlarging the
+ *           blur.
  *
  * Attributes of the -st-shadow property.
  */
@@ -26,6 +28,7 @@ struct _StShadow {
     gdouble      xoffset;
     gdouble      yoffset;
     gdouble      blur;
+    gdouble      spread;
 };
 
 GType     st_shadow_get_type (void) G_GNUC_CONST;
@@ -33,7 +36,8 @@ GType     st_shadow_get_type (void) G_GNUC_CONST;
 StShadow *st_shadow_new      (ClutterColor   *color,
                               gdouble         xoffset,
                               gdouble         yoffset,
-                              gdouble         blur);
+                              gdouble         blur,
+                              gdouble         spread);
 StShadow *st_shadow_copy     (const StShadow *shadow);
 void      st_shadow_free     (StShadow       *shadow);
 
